// CEnvLayer.cpp : Implementation of CEnvLayer

//

// 11/19/03 Rich Williams - modified for use in SEEK

//



#include "EnvLayer.h"

#include "ByteMatrix.h"

#include "Utilities.h"



#include <math.h>

#include <string.h>

#include <stdio.h>

#include <stdlib.h>

#include <fstream>

#include <cstdlib>

#include <iostream>



#include <scew.h>



#include "gioptr.h"



// GridIOLibrary global object

extern GridIOLibrary gridio;



// global color scheme set for use with EnvLayer objects

#ifdef __GARPWIN

  #include "ColorScheme.h"

  ColorSchemes oColorSchemes;

#endif



// ================================================================================

// ================================================================================

// ================================================================================

EnvLayer::EnvLayer()

{

  strcpy(strId, "");

  strcpy(strTitle, "");

  strcpy(strFilename, "");

  strcpy(strMatrixFilename, "");

  strcpy(strMatrixType, "");

  strcpy(strDescription, "");



  strcpy(strValueUnits, "");

  strcpy(strMapUnits, "");

  strcpy(strCoordSys, "");

  strcpy(strLayerType, "");



  dblXMin = dblXMax = dblYMin = dblYMax = dblCellSize =

    dblMinValue = dblMaxValue = dblValueCoef = dblMissingValue = 0.0;



  intRows = intColumns = intCells = 0;



  blnReadOnly = true;



  objMatrix = NULL;

}



// ================================================================================

EnvLayer::EnvLayer(const EnvLayer& objEnvLayer)

{

  strcpy(strId, objEnvLayer.strId);

  strcpy(strTitle, objEnvLayer.strTitle);

  strcpy(strFilename, objEnvLayer.strFilename);

  strcpy(strMatrixFilename, objEnvLayer.strMatrixFilename);

  strcpy(strMatrixType, objEnvLayer.strMatrixType);



  strcpy(strValueUnits, objEnvLayer.strValueUnits);

  strcpy(strMapUnits, objEnvLayer.strMapUnits);

  strcpy(strCoordSys, objEnvLayer.strCoordSys);

  strcpy(strLayerType, objEnvLayer.strLayerType);



  dblXMin = objEnvLayer.dblXMin;

  dblXMax = objEnvLayer.dblXMax;

  dblYMin = objEnvLayer.dblYMin;

  dblYMax = objEnvLayer.dblYMax;

  dblCellSize = objEnvLayer.dblCellSize;

  dblMinValue = objEnvLayer.dblMinValue;

  dblMaxValue = objEnvLayer.dblMaxValue;

  dblValueCoef = objEnvLayer.dblValueCoef;

  dblMissingValue = objEnvLayer.dblMissingValue;



  intRows = objEnvLayer.intRows;

  intColumns = objEnvLayer.intColumns;

  intCells = intRows * intColumns;



  blnReadOnly = objEnvLayer.blnReadOnly;



  if (objEnvLayer.objMatrix)

    objMatrix = objEnvLayer.objMatrix->clone();

  else

    objMatrix = NULL;

}



// ================================================================================

EnvLayer::~EnvLayer()

{

  if (objMatrix)

    delete (objMatrix);

}



// ================================================================================

bool EnvLayer::isGridIOLibPresent()

{

  return gridio.isLibPresent();

}



typedef int (*GridCopyFuncType)(char *, char *);

typedef int (*GridIOSetupFuncType)(void);

// ================================================================================

void EnvLayer::copyGrid(char * sSource, char * sDest)

{

  throw GarpException(2001, "EnvLayer::copyGrid() method is deprecated.");



  /*

    char name1[STRLEN], name2[STRLEN];

  int iRes, iLen;



  //strcpy(sSource, "E:\\Proj\\KDI\\DesktopGarp\\Experiments\\BestSubsetsTest\\grid00000\\task_000001_0");

  //strcpy(sDest, "E:\\Proj\\KDI\\DesktopGarp\\Experiments\\BestSubsetsTest\\BestSubsets\\Acodas\\proj00\\task_000001_0");



  if ((strlen(sSource) >= STRLEN) || (strlen(sDest) >= STRLEN))

    throw GarpException(1001, "CopyGrid(): At least one of input file names is too big");



  iLen = strlen(sDest);



  strcpy(name1, sSource);

  strcpy(name2, sDest);



  GridIOSetupFuncType fs;

  int result;



    HINSTANCE hinstLib;



  hinstLib = LoadLibrary("avgridio");

  fs = (GridIOSetupFuncType) GetProcAddress(hinstLib, "GridIOSetup");

  result = fs();



  GridCopyFuncType f;



  f = (GridCopyFuncType) GetProcAddress(hinstLib, "GridCopy");

  iRes = f(name1, name2);



  FreeLibrary(hinstLib);

  */



  /*

  if (!isGridIOLibPresent())

    throw GarpException(1002, "CopyGrid(): Could not load AVGRIDIO.DLL");



  if (!gridio.CellLyrExists(name1))

    throw GarpException(1003, "CopyGrid(): Input grid not found");



  if (gridio.CellLyrExists(name2))

    gridio.GridKill(name2, 1);



  iRes = gridio.GridCopy(name1, name2);

  */



  return;

}



// ================================================================================

void EnvLayer::asSpecies()

{

  dblMinValue =   1;

  dblMaxValue = 254;

  dblValueCoef = 1.0;

  dblMissingValue = -9999.0;



  strcpy(strLayerType, "Species");

}



// ================================================================================

void EnvLayer::set(int intPos, BYTE value)

{

  //try

  {

    objMatrix->set(intPos, value);

  }

  //catch (...)

  {

  //	throw GarpException(2, "Could not set value in byte matrix: matrix is possibly not set");

  }

}



// ================================================================================

BYTE EnvLayer::get(int intPos)

{

  try

  {

    return objMatrix->get(intPos);

  }



  catch (...)

  {

    throw GarpException(2, "Could not get value from byte matrix: matrix is possibly not set");

  }

}



// ================================================================================

int EnvLayer::scale(double dblUnscaledValue)

{

  int b;



  if (dblValueCoef != 0)

    b = (int)( ( (dblUnscaledValue - dblMinValue) / dblValueCoef ) + MIN_SCALED_VALUE);

  else

    b = 0;



  return b;

}



// ================================================================================

double EnvLayer::unscale(int bytScaledValue)

{

  double v;

  v = ( ( (double) bytScaledValue - MIN_SCALED_VALUE) * dblValueCoef ) + dblMinValue;

  return v;

}



// ================================================================================

void EnvLayer::rewind()

{

  if (objMatrix)

    objMatrix->rewind();

  else

    throw GarpException(1, "Byte matrix not defined for this EnvLayer");

}



// ================================================================================

char * EnvLayer::getId()

{

  char * id = new char[256];



  if (strcmp(strId, "NULL") != 0)

    return strcpy(id, strId);

  else

    return strcpy(id, "");

}



// ================================================================================

char * EnvLayer::getValueUnits()

{

  char * units = new char[256];

  if (strcmp(strValueUnits, "NULL") != 0)

    return strcpy(units, strValueUnits);

  else

    return strcpy(units, "");

}



// ================================================================================

void EnvLayer::metadataFromEnvLayer(EnvLayer& fromLayer)

{

  // copy metadata from other env layer to this env layer

  // but dont touch information on filenames and byte matrix

  strcpy(strId, fromLayer.strId);

  strcpy(strTitle, fromLayer.strTitle);

}



// ================================================================================

void EnvLayer::copyFromEnvLayer(EnvLayer& fromLayer)

{

  // copy the contents and metadata from other envlayer to this env layer

  // but dont touch information on filenames and byte matrix



  // first copy metadata

  metadataFromEnvLayer(fromLayer);

}



// ================================================================================

void EnvLayer::metadataFromArcGrid(char * strArcGrid)

{

  int iType, iChannel, iRes;

  double dBndBox[4], dNewBndBox[4];

  //float fAux;



  // create byte matrix and leave it open

  setMatrix(true);



  if (!gridio.isLibPresent())

    throw GarpException(1, "ESRI Grid IO library could not be loaded. Arc/Info Grid support disabled.");



  gridio.Initialize();



  // open file, but kill it before

  iChannel = gridio.CellLayerOpen(strArcGrid, READONLY, CELLIO, &iType, &dblCellSize);

  iRes = gridio.BndCellRead(strArcGrid, dBndBox);



  // get min and max values

  iRes = gridio.StaGetMinmaxDbl(strArcGrid, &dblMinValue, &dblMaxValue);



  // get layer bounding box

  // order of values is wrong: correct position would be: dblXMin, dblYMin, dblXMax, dblYMax;

  // let's check if there someone using this method. No one should.



  dblXMin = dNewBndBox[0] = dBndBox[0];

  dblXMax = dNewBndBox[1] = dBndBox[1];

  dblYMin = dNewBndBox[2] = dBndBox[2];

  dblYMax = dNewBndBox[3] = dBndBox[3];



  iRes = gridio.AccessWindowSet(dBndBox, dblCellSize, dNewBndBox);



  // number of cols and rows

  intColumns = gridio.WindowCols();

  intRows    = gridio.WindowRows();



  /*

  iRes = gridio.GetMissingFloat(&fAux);

  dblMissingValue = (double) fAux;

  */

  dblMissingValue = -9999.0;



  // get the number of cells

  intCells = intRows * intColumns;



  dblValueCoef = (dblMaxValue - dblMinValue) / SCALED_VALUE_RANGE;



  // close file after reading

  iRes = gridio.AccessWindowClear();

  iRes = gridio.CellLyrClose(iChannel);



  gridio.Release();

}



// ================================================================================

void EnvLayer::metadataFromAsciiGrid(char * strAsciiGrid)

{

  FILE *inf;

  char S[256];

  int i;

  double dblValue;



  // open input file

  inf = fopen(strAsciiGrid, "r");



  if (inf)

  {

    char msg[256];

    sprintf(msg, "File <%s> contains an invalid Ascii Grid", strAsciiGrid);



    // find Asc Grid Parameters

    dblMinValue =  9999.0;

    dblMaxValue = -9999.0;



    fscanf(inf,"%s %d", S, &intColumns);

    if (strcmp(S, "ncols")) throw GarpException(1, msg);



    fscanf(inf,"%s %d", S, &intRows);

    if (strcmp(S, "nrows")) throw GarpException(1, msg);



    fscanf(inf,"%s", S);

    dblXMin = GarpUtil::readDoubleFromFile(inf);

    if (strcmp(S, "xllcorner")) throw GarpException(1, msg);



    fscanf(inf,"%s", S);

    dblYMin = GarpUtil::readDoubleFromFile(inf);

    if (strcmp(S, "yllcorner")) throw GarpException(1, msg);



    fscanf(inf,"%s", S);

    dblCellSize = GarpUtil::readDoubleFromFile(inf);

    if (strcmp(S, "cellsize")) throw GarpException(1, msg);



    fscanf(inf,"%s", S);

    dblMissingValue = GarpUtil::readDoubleFromFile(inf);



    // get the number of cells

    intCells = intRows * intColumns;



    dblXMax = dblXMin + ((double) intColumns * dblCellSize);

    dblYMax = dblYMin + ((double) intRows    * dblCellSize);



    // finds min and max values for the Ascii Grid

    for (i = 0; i < intCells; i++)

    {

      if (feof(inf))

      {

        sprintf(msg, "File <%s> contains less cells (%d) than expected (%d)", strAsciiGrid, i, intCells);

        throw GarpException(1, msg);

      }



      dblValue = GarpUtil::readDoubleFromFile(inf);



      if (dblValue != dblMissingValue)

      {

        if (dblMinValue > dblValue)

          dblMinValue = dblValue;



        if (dblMaxValue < dblValue)

          dblMaxValue = dblValue;

      }

    }



    dblValueCoef = (dblMaxValue - dblMinValue) / SCALED_VALUE_RANGE;



    // close file after reading the header

    fclose(inf);

  }

  else

  {

    char msg[256];

    sprintf(msg, "Ascii raster grid <%s> not found", strAsciiGrid);

    throw GarpException(1, msg);

  }

}



// ================================================================================

void EnvLayer::fromArcGrid(char * strArcGrid)

{

  float dblValue;

  int intValue;

  BYTE val;

  int iType, iChannel, iRes;

  double dBndBox[4], dNewBndBox[4];

  int i, row, col;



  // read metadata from ascii grid

  metadataFromArcGrid(strArcGrid);



  // create byte matrix and leave it open

  setMatrix(true);



  if (!gridio.isLibPresent())

    throw GarpException(1, "ESRI Grid IO library could not be loaded. Arc/Info Grid support disabled.");



  //added by DV

  gridio.Initialize();



  // open file, but kill it before

  iChannel = gridio.CellLayerOpen(strArcGrid, READONLY, CELLIO, &iType, &dblCellSize);

  iRes = gridio.BndCellRead(strArcGrid, dBndBox);



  iRes = gridio.WindowCols();

  iRes = gridio.WindowRows();



  // get layer bounding box

  dNewBndBox[0] = dBndBox[0];

  dNewBndBox[1] = dBndBox[1];

  dNewBndBox[2] = dBndBox[2];

  dNewBndBox[3] = dBndBox[3];



  iRes = gridio.AccessWindowSet(dBndBox, dblCellSize, dNewBndBox);



  if (iChannel >= 0)

  {

    if (iType == CELLINT)

    {

      // grid is type INT

      // write data to matrix

      for (i = 0; i < intCells; i++)

      {

        row = i / intColumns;

        col = i % intColumns;



        iRes = gridio.GetWindowCellInt(iChannel, col, row, &intValue);



        if (intValue == (int) dblMissingValue)

          val = MASK_VALUE;

        else

          val = scale((double) intValue);



        objMatrix->set(i, val);

      }

    }



    else

    {

      // grid is type FLOAT

      // write data to matrix

      for (i = 0; i < intCells; i++)

      {

        row = i / intColumns;

        col = i % intColumns;



        iRes = gridio.GetWindowCellFloat(iChannel, col, row, &dblValue);



        if (GarpUtil::equalEps(dblValue, dblMissingValue, EPS))

          val = MASK_VALUE;

        else

          val = scale(dblValue);



        objMatrix->set(i, val);

      }

    }





    iRes = gridio.AccessWindowClear();

    iRes = gridio.CellLyrClose(iChannel);



    objMatrix->save();



    //added by DV

    gridio.Release();

  }



  else

  {

    gridio.Release();

    char msg[256];

    sprintf(msg, "ArcGrid <%s> not found", strArcGrid);

    throw GarpException(1, msg);

  }

}



// ================================================================================

void EnvLayer::fromAsciiGrid(char * strAsciiGrid)

{

  char S[256];

  int dummy, i;

  FILE * inf;

  double dblValue;

  BYTE val;



  // read metadata from ascii grid

  metadataFromAsciiGrid(strAsciiGrid);



  // create byte matrix and leave it open

  setMatrix(true);



  // open input and output files

  inf = fopen(strAsciiGrid, "r");



  if (inf)

  {

    // jump over the reader

    fscanf(inf,"%s %d", S, &dummy);

    fscanf(inf,"%s %d", S, &dummy);

    fscanf(inf,"%s", S); GarpUtil::readDoubleFromFile(inf);

    fscanf(inf,"%s", S); GarpUtil::readDoubleFromFile(inf);

    fscanf(inf,"%s", S); GarpUtil::readDoubleFromFile(inf);

    fscanf(inf,"%s %d", S, &dummy);



    // write data to matrix

    for (i = 0; i < intCells; i++)

    {

      dblValue = GarpUtil::readDoubleFromFile(inf);



      if (dblValue == dblMissingValue)

        val = MASK_VALUE;

      else

        val = scale(dblValue);



      objMatrix->set(i, val);

    }



    fclose(inf);



    objMatrix->save();

  }



  else

  {

    char msg[256];

    sprintf(msg, "Ascii raster grid <%s> not found", strAsciiGrid);

    throw GarpException(1, msg);

  }

}



// ================================================================================

void EnvLayer::toArcGrid(char * strArcGrid)

{

  int iChannel, iRes;

  double dBndBox[4], dNewBndBox[4];

  BYTE scaledValue;

    int unscaledValue;

  int pos, row, col;



  if (!gridio.isLibPresent())

    throw GarpException(1, "ESRI Grid IO library could not be loaded. Arc/Info Grid support disabled.");



  //added by DV

  gridio.Initialize();



  // get layer bounding box

  dNewBndBox[0] = dBndBox[0] = dblXMin;

  dNewBndBox[1] = dBndBox[1] = dblYMin;

  dNewBndBox[2] = dBndBox[2] = dblXMax;

  dNewBndBox[3] = dBndBox[3] = dblYMax;



  // open file, but kill it before

  iRes = gridio.GridKill(strArcGrid, 1);

  iChannel = gridio.CellLayerCreate(strArcGrid, WRITEONLY, CELLIO, CELLINT, dblCellSize, dBndBox);

  iRes = gridio.AccessWindowSet(dBndBox, dblCellSize, dNewBndBox);



  if (iChannel >= 0)

  {

    // initializes the position counter

    pos = 0;



    // go through the layer cells

    for (row = 0; row < intRows; row++)

    {

      for (col = 0; col < intColumns; col++)

      {

        scaledValue = objMatrix->get(pos++);



        if (scaledValue != MASK_VALUE)

        {

          if (scaledValue == MISSING_VALUE)

            scaledValue = ABSENCE;



          unscaledValue = (int) unscale(scaledValue);

          gridio.PutWindowCellInt(iChannel, col, row, unscaledValue);

        }

        else

        {

          gridio.PutWindowCellInt(iChannel, col, row, MISSINGINT);

        }

      }

    }



    iRes = gridio.AccessWindowClear();

    iRes = gridio.CellLyrClose(iChannel);



    //added by DV

    gridio.Release();

  }

  else

  {

    //added by DV

    gridio.Release();

    char msg[256];

    sprintf(msg, "Arc/Info grid <%s> could not be opened for writing", strArcGrid);

    throw GarpException(1, msg);

  }

}



// ================================================================================

void EnvLayer::toAsciiGrid(char * strAsciiGrid)

{

  FILE * gridf;

  BYTE scaledValue;

    int unscaledValue;

  int pos, row, col;



  // open file

  gridf = fopen(strAsciiGrid, "w");



  if (gridf)

  {

    // write header

    fprintf(gridf, "ncols %d\n",        intColumns);

    fprintf(gridf, "nrows %d\n",        intRows);

    fprintf(gridf, "xllcorner %f\n",    dblXMin);

    fprintf(gridf, "yllcorner %f\n",    dblYMin);

    fprintf(gridf, "cellsize %f\n",     dblCellSize);

    fprintf(gridf, "NODATA_value %d\n", (int) dblMissingValue);



    // initializes the position counter

    pos = 0;



    // go through the layer file

    for (row = 0; row < intRows; row++)

    {

      for (col = 0; col < intColumns; col++)

      {

        scaledValue = objMatrix->get(pos++);



        switch (pos)

        {

        case 363621:

        case 374345:

        case 366760:

        case 76564:

          int i = 10;

          break;

        }



        if (scaledValue != MASK_VALUE)

          unscaledValue = (int) unscale(scaledValue);

        else

          unscaledValue = (int) dblMissingValue;



        fprintf(gridf, "%d ", unscaledValue);

      }



      // put the end of line

      fprintf(gridf, "\n");

    }



    fclose(gridf);

  }

  else

  {

    char msg[256];

    sprintf(msg, "Ascii raster grid <%s> could not be opened for writing", strAsciiGrid);

    throw GarpException(1, msg);

  }

}



// ================================================================================

void EnvLayer::toBitmap(char * strBitmap)

{

  const BYTE oBitmapHeaderTemplate[118] =

  {

    // file header

    'B', 'M', 0xFF, 0xFF, 0xFF, 0xFF,

    0x00, 0x00, 0x00, 0x00,

    0x76, 0x00, 0x00, 0x00,



    // info header

    0x28, 0x00, 0x00, 0x00,

    0xFF, 0xFF, 0xFF, 0xFF,

    0xFF, 0xFF, 0xFF, 0xFF,

    0x01, 0x00, 0x04, 0x00,

    0x00, 0x00, 0x00, 0x00,

    0x00, 0x00, 0x00, 0x00,

    0x00, 0x00, 0x00, 0x00,

    0x00, 0x00, 0x00, 0x00,

    0x00, 0x00, 0x00, 0x00,

    0x00, 0x00, 0x00, 0x00,



    // color pallete

    0x00, 0x00, 0x00, 0x00,

    0x00, 0x00, 0x80, 0x00,

    0x00, 0x80, 0x00, 0x00,

    0x00, 0x80, 0x80, 0x00,



    0x80, 0x00, 0x00, 0x00,

    0x80, 0x00, 0x80, 0x00,

    0x80, 0x80, 0x00, 0x00,

    0x80, 0x80, 0x80, 0x00,



    0xC0, 0xC0, 0xC0, 0x00,

    0x00, 0x00, 0xFF, 0x00,

    0x00, 0xFF, 0x00, 0x00,

    0x00, 0xFF, 0xFF, 0x00,



    0xFF, 0x00, 0x00, 0x00,

    0xFF, 0x00, 0xFF, 0x00,

    0xFF, 0xFF, 0x00, 0x00,

    0xFF, 0xFF, 0xFF, 0x00,

  };



  FILE * gridf;

  BYTE scaledValue, pixelValue;

  int pos, row, col, i, iAdjustValue;

  DWORD lFileSize, lWidth, lHeight;

  int size;



  // set adjust value for the bitmap

  iAdjustValue = (4 - (intColumns % 4)) % 4;



  // open file

  gridf = fopen(strBitmap, "wb");



  if (gridf)

  {

    // prepare header variables

    lWidth = intColumns * 2;

    lHeight = intRows * 2;

    lFileSize = (lWidth * lHeight) + 118;



    // write bitmap header

    BYTE oHeader[118];

    memcpy(oHeader, oBitmapHeaderTemplate, 118);

    memcpy(&oHeader[2],  &lFileSize, 4);

    memcpy(&oHeader[18], &lWidth, 4);

    memcpy(&oHeader[22], &lHeight, 4);



    size = fwrite(oHeader, 1, 118, gridf);



    // initializes the position counter

    pos = 0;



    // go through the layer file

    for (row = 0; row < intRows; row++)

    {

      for (i = 0; i < 2; i++)

        for (col = 0; col < intColumns + iAdjustValue; col++)

        {

          if (col < intColumns)

          {

            pos = col + (intRows - row - 1) * intColumns;



            // get value for the first four bits representing this pixel

            scaledValue = objMatrix->get(pos);



            switch (scaledValue)

            {

              case MASK_VALUE:    pixelValue = 0x00; break;

              case MISSING_VALUE: pixelValue = 0x22; break;

              case ABSENCE:       pixelValue = 0x88; break;

              case PRESENCE:      pixelValue = 0xAA; break;

              default:		    pixelValue = 0x00; break;

            }

          }

          else

            pixelValue = 0x00;



          // write byte representing 2 bits

          fputc(pixelValue, gridf);

        }

    }



    fclose(gridf);

  }

  else

  {

    char msg[256];

    sprintf(msg, "Bitmap file <%s> could not be opened for writing", strBitmap);

    throw GarpException(1, msg);

  }

}



// ================================================================================

void EnvLayer::fromDataPointStringSettingCounters(char * strDataPoints, EnvLayer * objMask,

      long& lMaskedPoints, long& lUnmaskedPoints, long& lUniqueMaskedPoints, long& lUniqueUnmaskedPoints)

{

  char * strAux, * strToken;

  double dblLong, dblLat;

  int intPos;



  lMaskedPoints = 0;

  lUnmaskedPoints = 0;

  lUniqueMaskedPoints = 0;

  lUniqueUnmaskedPoints = 0;



  strAux = NULL;



  // check if there is any data point

  if (!strDataPoints)

    throw GarpException(37, "No datapoint found.");



  // create byte matrix and leave it open

  setMatrix(true);



  // copy original datapoints in temp string variable

  strAux = new char[strlen(strDataPoints) + 2];

  strcpy(strAux, strDataPoints);



  try

  {

    strToken = strtok(strAux, " \n\t\r");

    while (strToken)

    {

      // get longitude from token that has already been read

      dblLong = atof(strToken);



      // read latitude from next token

      strToken = strtok(NULL, " \n\t\r");

      if (!strToken)

        throw GarpException(37, "Invalid datapoint file");

      else

        dblLat = atof(strToken);



      // get position in matrix

      intPos = getPosition(dblLong, dblLat);



      // count number of points in each class

      // check if point is inside the dataset boundary

      if (intPos == -1)

      {

        // point is masked

        lMaskedPoints++;

        lUniqueMaskedPoints++;

      }



      else // point is valid

      {

        BYTE iValue = objMatrix->get(intPos);

        BYTE iMask = objMask->get(intPos);



        if (iValue == PRESENCE)

        {

          // point is already in the matrix: it is a new point but not a new unique one

          if (iMask == MASK_VALUE)

            // point is masked

            lMaskedPoints++;

          else

            // point is unmasked

            lUnmaskedPoints++;

        }

        else

        {

          // point is new: it is a new unique point

          if (iMask == MASK_VALUE)

          {

            // point is masked

            lMaskedPoints++;

            lUniqueMaskedPoints++;

          }

          else

          {

            // point is unmasked

            lUnmaskedPoints++;

            lUniqueUnmaskedPoints++;

          }

        }

      }



      // if point is inside layer extent, set its value in layer

      if (intPos >= 0)

        objMatrix->set(intPos, PRESENCE);



      // get next token (longitude of next datapoint)

      strToken = strtok(NULL, " \n\t\r");

    }

  }



  catch (...)

  {

    if (strAux)

      delete strAux;

    throw GarpException(38, "Error reading datapoint string");

  }



  if (strAux)

    delete strAux;

}



// ================================================================================

void EnvLayer::fromDataPointString(char * strDataPoints)

{

  char * strAux, * strToken;

  double dblLong, dblLat;

  int intPos;



  strAux = NULL;



  // check if there is any data point

  if (!strDataPoints)

    throw GarpException(37, "No datapoint found.");



  // create byte matrix and leave it open

  setMatrix(true);



  // copy original datapoints in temp string variable

  strAux = new char[strlen(strDataPoints) + 2];

  strcpy(strAux, strDataPoints);



  try

  {

    strToken = strtok(strAux, " \n\t\r");

    while (strToken)

    {

      // get longitude from token that has already been read

      dblLong = atof(strToken);



      // read latitude from next token

      strToken = strtok(NULL, " \n\t\r");

      if (!strToken)

        throw GarpException(37, "Invalid datapoint file");

      else

        dblLat  = atof(strToken);



      // get position in matrix

      intPos = getPosition(dblLong, dblLat);



      // if point is inside layer extent, set its value in layer

      if (intPos >= 0)

        objMatrix->set(intPos, PRESENCE);



      // get next token (longitude of next datapoint)

      strToken = strtok(NULL, " \n\t\r");

    }

  }



  catch (...)

  {

    if (strAux)

      delete strAux;

    throw GarpException(38, "Error reading datapoint string");

  }



  if (strAux)

    delete strAux;

}



// ================================================================================

void EnvLayer::fromDataPoints(char * strDataPoints)

{

  throw GarpException(1, "The method EnvLayer::fromDataPoints is deprecated.");



  double dblLong, dblLat;

  int intPos;

  FILE * inf;



  // create byte matrix and leave it open

  setMatrix(true);



  inf = fopen(strDataPoints, "r");



  if (inf)

  {

    while (!feof(inf))

    {

      dblLong = GarpUtil::readDoubleFromFile(inf);

      dblLat  = GarpUtil::readDoubleFromFile(inf);



      // get position in matrix

      intPos = getPosition(dblLong, dblLat);



      // if point is inside layer extent, set its value in layer

      if (intPos >= 0)

        objMatrix->set(intPos, PRESENCE);

    }

  }

  else

  {

    char msg[256];

    sprintf(msg, "Data point file <%s> not found", strDataPoints);

    throw GarpException(1, msg);

  }

}



// ================================================================================

char * EnvLayer::descriptionXML(const char strType[])

{

  const int iStringFormatLen = 1024;

  const int iStringResultLen = 8 * 1024;



  char strFormat[iStringFormatLen];

  char * strResult = new char[iStringResultLen];



  strcpy(strFormat, "<EnvLayer Type=\"%s\" Id=\"%s\" Title=\"%s\" MatrixType=\"%s\" MatrixFileName=\"%s\" ");

  strcat(strFormat, " ValueUnits=\"%s\" MapUnits=\"%s\" CoordSys=\"%s\" LayerType=\"%s\"");

  strcat(strFormat, " Rows=\"%d\" Columns=\"%d\" XMin=\"%f\" XMax=\"%f\" YMin=\"%f\" YMax=\"%f\"");

  strcat(strFormat, " CellSize=\"%f\" MinValue=\"%f\" MaxValue=\"%f\" ValueCoef=\"%f\" />");



  if (strlen(strFormat) > iStringFormatLen)

    throw GarpException(82, "String <strFormat> size exceeded in EnvLayer::descriptionXML()");



  sprintf(strResult, strFormat, strType, strId, strTitle, strMatrixType, strMatrixFilename,

             strValueUnits, strMapUnits, strCoordSys, strLayerType,

             intRows, intColumns, dblXMin, dblXMax, dblYMin, dblYMax,

             dblCellSize, dblMinValue, dblMaxValue, dblValueCoef);



  if (strlen(strResult) > iStringResultLen)

    throw GarpException(82, "String <strResult> size exceeded in EnvLayer::descriptionXML()");



  return strResult;

}



// ================================================================================

char * EnvLayer::getDescriptionText()

{

  char * strResult = new char[1024];

  sprintf(strResult, "%s %s %f %f", strId, strFilename, dblMinValue, dblMaxValue);

  return strResult;

}



// ================================================================================

bool EnvLayer::isCompatible(EnvLayer * objAnotherEnvLayer)

{

  return ( (dblXMin == objAnotherEnvLayer->dblXMin) &&

       (dblXMax == objAnotherEnvLayer->dblXMax) &&

       (dblYMin == objAnotherEnvLayer->dblYMin) &&

       (dblYMax == objAnotherEnvLayer->dblYMax) &&

       (dblCellSize == objAnotherEnvLayer->dblCellSize) &&

       (intRows == objAnotherEnvLayer->intRows) &&

       (intColumns == objAnotherEnvLayer->intColumns) );

}



// ==================================================

//  Method:		int getCoordinates(pos, x, y)

// --------------------------------------------------

//  Purpose: Calculate the (x, y) coordinate corresponding

//           to the cell index in the grid array given by <pos>

//  Input:   <pos>: cell index in the grid array

//			 <x>, <y>: Variables where the coordinate values

//             will be stored

//	Output:	 Pair of values <x>, <y> for the coordinate

//             corresponding to the <pos> grid cell

//  Remarks: The method getPosition does the opposite

// --------------------------------------------------

void EnvLayer::getCoordinates(long pos, double& dblX, double& dblY)

{

  dblY = dblYMax - ((double)(pos / intColumns) * dblCellSize);

  dblX = ((double)(pos % intColumns) * dblCellSize) + dblXMin;

}



// ==================================================

//  Method:		long getPosition(x, y)

// --------------------------------------------------

//  Purpose: Calculate the cell index in the grid array

//           that corresponds to the (x, y) coordinate given

//  Input:   Pair (x, y) corresponding to a coordinate

//  Output:  The index of the (x, y) point in the grid array

//  Remarks: The method getCoordinates does the opposite

// --------------------------------------------------

long EnvLayer::getPosition(double dblX, double dblY)

{

  int pos, f1, f2;

    double px, py;



  if (!(GarpUtil::between(dblX, dblXMin, dblXMax) &&

      GarpUtil::between(dblY, dblYMin, dblYMax)))

  {

    // point out of spatial range

    return -1;

  }



  else

  {

    // it is inside spatial range

    px = fabs((dblX - dblXMin) / (dblXMax - dblXMin));

    py = fabs((dblY - dblYMax) / (dblYMax - dblYMin));



    f1 = (int) (px * (double) intColumns);

    f2 = (int) (py * (double) intRows);



    // debug

    pos = f2 * intColumns + f1;



    //if (Verboseflag)

    //	printf("\nx %f y %f xi:%d yi:%d %d frac: %5.2f %5.2f", dblX, dblY, f1, f2, pos, px, py);



    if (pos >= intCells || pos < 0 )

    {

      // point out of spatial range

      return -1;

    }



    else

      return pos;

  }

}



// ================================================================================

char * EnvLayer::toXML(const char strType[])

{

  /*

    <EnvLayer Id="US_4km" Title="US_4km" Filename="US_4km.env"

        Rows="623" Columns="1465"

        XMin="-126.020833" XMax="-64.979167"

        YMin="24.062500" YMax="50.020833" CellSize="0.041667"

        MapUnits="NULL" CoordSys="NULL">

    </EnvLayer>

  */



  const int iStringTagLen = 2 * 1024;

  const int iXMLStringLen = 8 * 1024;



  char * strXML, strSpatialTag[iStringTagLen], strMatrixTag[iStringTagLen];



  strXML = new char[iXMLStringLen];



  // create the ByteMatrix tag

  if (objMatrix)

    strcpy(strMatrixTag, objMatrix->toXML());

  else

    strcpy(strMatrixTag, "<ByteMatrix/>");



  // create the Spatial tag

  sprintf(strSpatialTag, "Rows=\"%d\" Columns=\"%d\"\n XMin=\"%f\" XMax=\"%f\" YMin=\"%f\" YMax=\"%f\" CellSize=\"%f\"\n MinValue=\"%f\" MaxValue=\"%f\" ValueCoef=\"%f\" MissingValue=\"%f\"\n MapUnits=\"%s\" CoordSys=\"%s\"",

    intRows, intColumns, dblXMin, dblXMax, dblYMin, dblYMax, dblCellSize, dblMinValue, dblMaxValue, dblValueCoef, dblMissingValue, strMapUnits, strCoordSys);



  sprintf(strXML, "<EnvLayer Type=\"%s\" Id=\"%s\" Title=\"%s\" Filename=\"%s\"\n ValueUnits=\"%s\" LayerType=\"%s\">\n  %s\n  %s\n</EnvLayer>",

    strType, strId, strTitle, strFilename, strValueUnits, strLayerType, strSpatialTag, strMatrixTag);



  if ((strlen(strSpatialTag) > iStringTagLen) ||

    (strlen(strMatrixTag)  > iStringTagLen) ||

    (strlen(strXML)        > iXMLStringLen))

    throw GarpException(82, "String size exceeded in EnvLayer::toXML()");



  return strXML;

}



// ================================================================================

void EnvLayer::readXMLAttributes(scew_element const * pXMLSetNode)

{

  char * sName, * sValue;

  int i, n;



  scew_attribute * pXMLAttr;



  if (pXMLSetNode)

  {

    // get attribute list

    n = scew_attribute_count(pXMLSetNode);



    // process each attribute

    for (i = 0; i < n; i++)

    {

      pXMLAttr = scew_attribute_by_index(pXMLSetNode, i);



      sName = (char *) scew_attribute_name(pXMLAttr);

      sValue = (char *) scew_attribute_value(pXMLAttr);;



      if      (strcmp(sName, "Id") == 0)				strcpy(strId,          sValue);

      else if (strcmp(sName, "Title") == 0)			strcpy(strTitle,       sValue);

      else if (strcmp(sName, "Filename") == 0)		strcpy(strFilename,    sValue);

      else if (strcmp(sName, "Description") == 0)		strcpy(strDescription, sValue);

      else if (strcmp(sName, "MapUnits") == 0)		strcpy(strMapUnits,    sValue);

      else if (strcmp(sName, "CoordSys") == 0)		strcpy(strCoordSys,    sValue);

      else if (strcmp(sName, "ValueUnits") == 0)		strcpy(strValueUnits,  sValue);

      else if (strcmp(sName, "LayerType") == 0)		strcpy(strLayerType,   sValue);

      else if (strcmp(sName, "MatrixType") == 0)		strcpy(strMatrixType,   sValue);

      else if (strcmp(sName, "MatrixFileName") == 0)	strcpy(strMatrixFilename,   sValue);



      else if (strcmp(sName, "Rows") == 0)		intRows     = atoi(sValue);

      else if (strcmp(sName, "Columns") == 0)		intColumns  = atoi(sValue);

      else if (strcmp(sName, "XMin") == 0)		dblXMin     = atof(sValue);

      else if (strcmp(sName, "XMax") == 0)		dblXMax     = atof(sValue);

      else if (strcmp(sName, "YMin") == 0)		dblYMin     = atof(sValue);

      else if (strcmp(sName, "YMax") == 0)		dblYMax     = atof(sValue);

      else if (strcmp(sName, "MinValue") == 0)	dblMinValue = atof(sValue);

      else if (strcmp(sName, "MaxValue") == 0)	dblMaxValue = atof(sValue);

      else if (strcmp(sName, "CellSize") == 0)	dblCellSize = atof(sValue);

    }



    dblValueCoef = (dblMaxValue - dblMinValue) / SCALED_VALUE_RANGE;



    intCells = intRows * intColumns;

  }



  else

    throw(1004, "Cannot read attributes of EnvLayer. XML node is empty.");



}

// ================================================================================

void EnvLayer::fromXML(scew_element const * pXMLNode)

{



  if (pXMLNode)

  {

    // parse EnvLayerSet node

    // read attributes

    readXMLAttributes(pXMLNode);

  }

  else

    throw GarpException(26, "XML source does not contain an EnvLayer node.");



}



// ================================================================================

void EnvLayer::fromXML(char * strXML)

{

  int iResult;



    scew_tree* oTree = NULL;

    scew_parser* oParser = NULL;



  scew_element * pXMLSetNode, * pXMLMatrixNode;



  oParser = scew_parser_create();



    // Loads an XML file

    iResult = scew_parser_load_buffer(oParser, (const char *) strXML, strlen(strXML));



  if (iResult)

  {

    // load data from XML Node

    pXMLSetNode = scew_element_by_name_including_parent(scew_tree_root(oTree), "EnvLayer");



    if (pXMLSetNode)

    {

      // parse EnvLayerSet node

      // read attributes

      readXMLAttributes(pXMLSetNode);



      pXMLMatrixNode = scew_element_by_name_including_parent(pXMLSetNode, "ByteMatrix");



      if (pXMLMatrixNode)

      { objMatrix = ByteMatrix::fromXML(pXMLMatrixNode); }



      // Remember to free tree (scew_parser_free does not free it)

      // Frees the SCEW parser

      scew_tree_free(oTree);

      scew_parser_free(oParser);

    }

    else

    {

      // Remember to free tree (scew_parser_free does not free it)

      // Frees the SCEW parser

      scew_tree_free(oTree);

      scew_parser_free(oParser);



      throw GarpException(26, "XML source does not contain an EnvLayer node.");

    }



  }

  else

  {

    // Remember to free tree (scew_parser_free does not free it)

    // Frees the SCEW parser

    scew_tree_free(oTree);

    scew_parser_free(oParser);



    throw GarpException(22, "Could not load node from XML text.");

  }

}



// ================================================================================

void EnvLayer::loadData(int iLoadMode)

{

  // load mode equals to 0 or 1 don't affect the data load

  // value 1 where formely used for loading the additional layer data

  // that used to be stored in .env files.

  // it has become deprecated because all data for env layers is stored

  // on the EnvLayerSet xml files.

  // mode 2 force data load on memory (even if it is a on-disk matrix)

  // mode 3 force data load on memory-maped files (MMF) (even if it is a on-disk or on-memory matrix)



  if (iLoadMode == 2)

  {

    // force data load in memory (if it is a byte matrix in disk)

    // change type so data is loaded entirely in memory

    if (strcmp(strMatrixType, "RawByteMatrixInDisk") == 0)

      strcpy(strMatrixType, "RawByteMatrixInMemory");

  }



#ifdef __GARPWIN

  else if (iLoadMode == 3)

  {

    // force data load as memory-maped file (if it is a byte matrix in disk or in memory)

    // change type so it uses memory-maped files

    if ((strcmp(strMatrixType, "RawByteMatrixInDisk") == 0) ||

      (strcmp(strMatrixType, "RawByteMatrixInMemory") == 0))

      strcpy(strMatrixType, "RawByteMatrixInMMFile");

  }

#endif //__GARPWIN





  // check if there is information enough to open the matrix

  if (!((strcmp(strMatrixFilename, "") == 0) ||

      (strcmp(strMatrixType,     "") == 0)))

    openMatrix();



}



// ================================================================================

void EnvLayer::loadText(char * strLoadFilename, int iLoadMode)

{

  char S[256], S1[256];

  FILE * inf;



  // DEPRECATED

  throw GarpException(1, "The method EnvLayer::loadText() is deprecated now. Use EnvLayer::loadData() instead.");



  strcpy(strFilename, strLoadFilename);

  inf = fopen(strFilename, "r");



  if (inf)

  {

    fscanf(inf, "%s %s", S, strId);

    fscanf(inf, "%s %s", S, strTitle);



    // ignore saved filename

    // use parameter instead

    fscanf(inf, "%s %s", S, S1);



    fscanf(inf, "%s %s", S, strMatrixFilename);

    fscanf(inf, "%s %s", S, strMatrixType);



    if (iLoadMode == 1)

    {

      // change type so data is left on disk

      strcpy(strMatrixType, "RawByteMatrixInDisk");

    }

    else if (iLoadMode == 2)

    {

      // change type so data is loaded entirely in memory

      if (strcmp(strMatrixType, "RawByteMatrixInDisk") == 0)

        strcpy(strMatrixType, "RawByteMatrixInMemory");

    }

    else if (iLoadMode == 3)

    {

      // change type so it uses memory-maped files

      if ((strcmp(strMatrixType, "RawByteMatrixInDisk") == 0) ||

        (strcmp(strMatrixType, "RawByteMatrixInMemory") == 0))

        strcpy(strMatrixType, "RawByteMatrixInMMFile");

    }





    // check if there is information enough to open the matrix

    if (!((strcmp(strMatrixFilename, "") == 0) ||

        (strcmp(strMatrixType,     "") == 0)))

      openMatrix();



    fscanf(inf, "%s %s", S, strValueUnits);

    fscanf(inf, "%s %s", S, strMapUnits);

    fscanf(inf, "%s %s", S, strCoordSys);

    fscanf(inf, "%s %s", S, strLayerType);



    fscanf(inf, "%s", S); dblXMin = GarpUtil::readDoubleFromFile(inf);

    fscanf(inf, "%s", S); dblXMax = GarpUtil::readDoubleFromFile(inf);

    fscanf(inf, "%s", S); dblYMin = GarpUtil::readDoubleFromFile(inf);

    fscanf(inf, "%s", S); dblYMax = GarpUtil::readDoubleFromFile(inf);

    fscanf(inf, "%s", S); dblCellSize = GarpUtil::readDoubleFromFile(inf);

    fscanf(inf, "%s", S); dblMinValue = GarpUtil::readDoubleFromFile(inf);

    fscanf(inf, "%s", S); dblMaxValue = GarpUtil::readDoubleFromFile(inf);

    fscanf(inf, "%s", S); dblValueCoef = GarpUtil::readDoubleFromFile(inf);

    fscanf(inf, "%s", S); dblMissingValue = GarpUtil::readDoubleFromFile(inf);



    dblValueCoef = (dblMaxValue - dblMinValue) / SCALED_VALUE_RANGE;



    fscanf(inf, "%s %d", S, &intRows);

    fscanf(inf, "%s %d", S, &intColumns);



    intCells = intRows * intColumns;



    fclose(inf);

  }

  else

  {

    char msg[256];

    sprintf(msg, "Environmental layer metadata <%s> could not be opened for reading", strFilename);

    throw GarpException(1, msg);

  }

}



// ================================================================================

void EnvLayer::saveText(char * strSaveFilename)

{

  FILE * outf;



  if ((strcmp(strSaveFilename, "NULL") == 0) ||

    (strcmp(strSaveFilename, "") == 0))

    throw GarpException(2, "Cannot save dataset with an empty filename");



  strcpy(strFilename, strSaveFilename);

  outf = fopen(strFilename, "w");



  if (outf)

  {

    fprintf(outf, "%s %s\n", "Id", strId);

    fprintf(outf, "%s %s\n", "Title", strTitle);

    fprintf(outf, "%s %s\n", "Filename", strFilename);

    fprintf(outf, "%s %s\n", "MatrixFilename", strMatrixFilename);

    fprintf(outf, "%s %s\n", "MatrixType", strMatrixType);



    fprintf(outf, "%s %s\n", "ValueUnits", strValueUnits);

    fprintf(outf, "%s %s\n", "MapUnits", strMapUnits);

    fprintf(outf, "%s %s\n", "CoordSys", strCoordSys);

    fprintf(outf, "%s %s\n", "LayerType", strLayerType);



    fprintf(outf, "%s %f\n", "MinX", dblXMin);

    fprintf(outf, "%s %f\n", "MaxX", dblXMax);

    fprintf(outf, "%s %f\n", "MinY", dblYMin);

    fprintf(outf, "%s %f\n", "MaxY", dblYMax);

    fprintf(outf, "%s %f\n", "CellSize", dblCellSize);

    fprintf(outf, "%s %f\n", "MinValue", dblMinValue);

    fprintf(outf, "%s %f\n", "MaxValue", dblMaxValue);

    fprintf(outf, "%s %f\n", "ValueCoef", dblValueCoef);

    fprintf(outf, "%s %f\n", "MissingValue", dblMissingValue);



    fprintf(outf, "%s %d\n", "Rows", intRows);

    fprintf(outf, "%s %d\n", "Columns", intColumns);



    fclose(outf);



    if (objMatrix)

      objMatrix->save();

  }

  else

  {

    char msg[256];

    sprintf(msg, "Environmental layer metadata <%s> could not be opened for writing", strFilename);

    throw GarpException(1, msg);

  }

}



// ================================================================================

void EnvLayer::setMatrix(bool blnCreateNew)

{

  char msg[256];

  //this line is commented out because it was causing a buffer overflow.

  //I couldn't figure out why so i commented it out.

  //CB 01.07.04

  //sprintf(msg, "Matrix type <%s> is not implemented yet", strMatrixType);

  if (objMatrix)

    delete objMatrix;



  if (strcmp(strMatrixType, "RawByteMatrixInDisk") == 0)

  {

    // this object is stored on disk, so the filename must be set

    objMatrix = new RawByteMatrixInDisk();



    if (blnCreateNew)

    { ((RawByteMatrixInDisk *) objMatrix)->create(strMatrixFilename, intCells); }



    objMatrix->open(strMatrixFilename);

  }



  else if (strcmp(strMatrixType, "RawByteMatrixInMemory") == 0)

  {

    // there are two uses for this object

    // one is to serve as a read/write cache for the InDisk version

    // in this case, caller gives the filename to be loaded

    // however, if filename is not set, then it is assumed that this byte matrix

    // is volatile (won't be saved on disk)

    objMatrix = new RawByteMatrixInMemory();



    if (blnCreateNew)

    { ((RawByteMatrixInMemory *) objMatrix)->create(strMatrixFilename, intCells); }



    objMatrix->open(strMatrixFilename);

  }



#ifdef __GARPWIN

  else if (strcmp(strMatrixType, "RawByteMatrixInMMFile") == 0)

  {

    // this object is stored on disk, so the filename must be set

    objMatrix = new RawByteMatrixInMMFile();



    if (blnCreateNew)

    { ((RawByteMatrixInMMFile *) objMatrix)->create(strMatrixFilename, intCells); }



    objMatrix->open(strMatrixFilename);

  }

#endif //__GARPWIN



  else if (strcmp(strMatrixType, "BooleanByteMatrix") == 0)

  {

    objMatrix = new BooleanByteMatrix();

  }



  else if (strcmp(strMatrixType, "ArcGridByteMatrix") == 0)

  {

    throw GarpException(1, "The methods createMatrix and setMatrix cannot be used with ArcGridByteMatrix objects. Please use EnvLayer::useArcGridByteMatrix instead");

  }



  else if (strcmp(strMatrixType, "BitmapByteMatrix") == 0)

  {

    throw GarpException(1, "The methods createMatrix and setMatrix cannot be used with BitmapByteMatrix objects. Please use EnvLayer::useBitmapMatrix instead");

  }



  else if (strcmp(strMatrixType, "") == 0)

  {

    throw GarpException(1, "Cannot open matrix without defining its type");

  }



  else

  {

    sprintf(msg, "Matrix type <%s> unknown", strMatrixType);

    throw GarpException(1, msg);

  }

}



// ================================================================================

void EnvLayer::useArcGridByteMatrix(char * strFilenameAux, double dCellSizeAux,

                  double dXMaxAux, double dYMaxAux,

                  double dXMinAux, double dYMinAux,

                  double dMinValueAux, double dMaxValueAux)

{

#ifdef __GRIDIO

  if (gridio.isLibPresent())

  {

    //Added by DV

    gridio.Initialize();



    ArcGridByteMatrix * oArcMatrix = new ArcGridByteMatrix();



    oArcMatrix->create(strFilenameAux, dCellSizeAux,

              dXMaxAux, dYMaxAux, dXMinAux, dYMinAux,

              dMinValueAux, dMaxValueAux);



    objMatrix = (ByteMatrix *) oArcMatrix;



    //Added by DV

    gridio.Release();

  }

  else

#endif //__GRIDIO

    throw GarpException(1, "ESRI Grid IO Library could not be loaded. Arc/Info Grid support disabled. Please contact systems support personel to fix this problem.");

}



// ================================================================================

void EnvLayer::useBitmapMatrix(int iCDepth, int iColumns, int iRows)

{

#ifdef __GARPWIN

  objMatrix = new BitmapByteMatrix();



  ((BitmapByteMatrix *) objMatrix)->initialize(iCDepth, iColumns, iRows);



  ((RawByteMatrixInMemory *) objMatrix)->create("", intCells * iCDepth / 8);



  objMatrix->open(strMatrixFilename);

#endif

}



// ================================================================================

BYTE * EnvLayer::getMatrixBytes(int &iSize)

{

  BYTE * pBytes;



  if (strcmp(objMatrix->type(), "BitmapByteMatrix") != 0)

    throw GarpException(121, "Only layers using BitmapByteMatrix can use EnvLayer::getMatrixBytes method");



  pBytes = (((RawByteMatrixInMemory *) objMatrix)->getArrayPtr());

  iSize =  (((RawByteMatrixInMemory *) objMatrix)->size());



  return pBytes;

}



// ================================================================================

ByteMatrix * EnvLayer::getMatrix()

{

  return objMatrix;

}



// ================================================================================

void EnvLayer::createMatrix(char * strParamMatrixType, char * strParamMatrixFilename)

{

  // set obj properties using the arguments

  strcpy(strMatrixType, strParamMatrixType);

  strcpy(strMatrixFilename, strParamMatrixFilename);



  setMatrix(true);

}



// ================================================================================

void EnvLayer::openMatrix()

{

  setMatrix(false);

}





// ================================================================================

// ================================================================================

#ifdef __GARPWIN



// ================================================================================

void EnvLayer::transferBytesToBitmap(BYTE * pBytes, int iColorDepth)

{

  // transfer the layer bytes to the array of bytes, acording to the bitmap properties



  // note: byte array alignment depends on the color depth

  int i, j;

  long pos;

  BYTE val;

  DWORD offset;

  int iPixelWidth;

  ColorScheme * pScheme;

  RGBTRIPLE * pColor;



  iPixelWidth = iColorDepth / 8;



  // get color scheme based on layer type

  pScheme = oColorSchemes.getScheme(strLayerType);



  // iterate throug the cells



  if (iColorDepth == 8)

  {

    for (i = 0; i < intRows; i++)

      for (j = 0; j < intColumns; j++)

      {

        val = get(i * intColumns + j);



        pos = j + (intRows - i - 1) * intColumns;

        offset = pos;



        // get color from scheme for this value

        pColor = pScheme->getColor(val);



        // reduce color width to 5 bits

        pBytes[offset] = PALETTERGB(pColor->rgbtRed, pColor->rgbtGreen, pColor->rgbtBlue);

      }

  }



  else if (iColorDepth == 16)

  {

    for (i = 0; i < intRows; i++)

      for (j = 0; j < intColumns; j++)

      {

        val = get(i * intColumns + j);



        pos = j + (intRows - i - 1) * (intColumns + 1);

        offset = pos * iPixelWidth;



        WORD wColor = 0;



        // get color from scheme for this value

        pColor = pScheme->getColor(val);



        // reduce color width to 5 bits

        wColor += (pColor->rgbtRed   / 8) << 10;

        wColor += (pColor->rgbtGreen / 8) << 5;

        wColor += pColor->rgbtBlue   / 8;



        pBytes[offset + 0] = LOBYTE(wColor);

        pBytes[offset + 1] = HIBYTE(wColor);

      }

  }



  else if (iColorDepth == 24)

  {

    // BUG: not working properly: DEBUG ...



    for (i = 0; i < intRows; i++)

      for (j = 0; j < intColumns; j++)

      {

        val = get(i * intColumns + j);



        pos = j + (intRows - i - 1) * intColumns;

        offset = pos * iPixelWidth;



        pScheme->copyColor(val, (RGBTRIPLE*) (&pBytes[offset]));

      }

  }



  else if (iColorDepth == 32)

  {

    for (i = 0; i < intRows; i++)

      for (j = 0; j < intColumns; j++)

      {

        val = get(i * intColumns + j);



        pos = j + (intRows - i - 1) * intColumns;

        offset = pos * iPixelWidth;



        pScheme->copyColor(val, (RGBTRIPLE*) (&pBytes[offset]));

      }

  }



  else

  {

    for (i = 0; i < intRows; i++)

      for (j = 0; j < intColumns; j++)

      {

        val = get(i * intColumns + j);



        pos = j + (intRows - i - 1) * intColumns;

        offset = pos * iPixelWidth;



        pBytes[offset] = val;

      }

  }

}



// ================================================================================

void EnvLayer::transferBytesToDIB(BYTE * pBytes)

{

  // transfer the layer bytes to the array of bytes, acording to the bitmap properties



  // note: byte array alignment depends on the color depth

  int i, j, iAdjustValue;

  long pos;

  BYTE val;

  DWORD offset;

  ColorScheme * pScheme;



  // get color scheme based on layer type

  pScheme = oColorSchemes.getScheme(strLayerType);



  // get adjust value correspondent to the number of columns,

  // to adjust with the 4 byte bitmap alignment

  iAdjustValue = intColumns % 4;



  // iterate throug the cells

  for (i = 0; i < intRows; i++)

    for (j = 0; j < intColumns; j++)

    {

      val = get(i * intColumns + j);

      pos = j + (intRows - i - 1) * intColumns;

      offset = pos * 3 + ((intRows - i - 1) * iAdjustValue);



      pScheme->copyColor(val, (RGBTRIPLE*) (&pBytes[offset]));

    }

}



// ================================================================================

#endif //__GARPWIN



// ================================================================================

// ================================================================================



