// EnvLayer.h : Declaration of the CEnvLayer

#ifndef __ENVLAYER_H_
#define __ENVLAYER_H_

#include "ByteMatrix.h"
#include "Utilities.h"

#include <scew.h>

#ifdef __GARPWIN
  #include "ColorScheme.h"
#endif

// ============================================================================
//  Regular EnvLayer Class
// ============================================================================
class EnvLayer
{
private:
  friend class CEnvLayer;
  friend class CEnvLayerSet;
  friend class EnvLayerSet;

  char strId[CHAR_ARRAY_SIZE];
  char strTitle[CHAR_ARRAY_SIZE];
  char strFilename[CHAR_ARRAY_SIZE];
  char strMatrixFilename[CHAR_ARRAY_SIZE];
  char strMatrixType[CHAR_ARRAY_SIZE];
  char strDescription[16000];

  double dblXMin, dblXMax, dblYMin, dblYMax;
  double dblCellSize;
  double dblMinValue, dblMaxValue, dblValueCoef;
  double dblMissingValue;
  int intRows, intColumns, intCells;

  char strValueUnits[CHAR_ARRAY_SIZE];
  char strMapUnits[CHAR_ARRAY_SIZE];
  char strCoordSys[CHAR_ARRAY_SIZE];
  char strLayerType[CHAR_ARRAY_SIZE];

  bool blnReadOnly;

  ByteMatrix * objMatrix;

private:
  void setMatrix(bool blnCreateNew);

  void readXMLAttributes(scew_element const * pXMLNode);

public:
  // Constructors
  EnvLayer();
  EnvLayer(const EnvLayer& objEnvLayer);

  // Destructor
  virtual ~EnvLayer();

  static bool isGridIOLibPresent();
  static void copyGrid(char * sSource, char * sDest);

  void loadText(char * strFilename, int iLoadMode);
  void saveText(char * strFilename);

  void loadData(int iLoadMode);
  char * getDescriptionText();
  char * descriptionXML(const char strType[]);

  void fromDescriptionText();

  void fromXML(scew_element const * pXMLNode);
  void fromXML(char * strXML);
  char * toXML(const char strType[]);

  void set(int intPos, BYTE value);
  BYTE get(int intPos);

  void rewind();

  int scale(double dblUnscaledValue);
  double unscale(int intScaledValue);

  void getCoordinates(long intPos, double& dblX, double& dblY);
  long getPosition(double dblX, double dblY);

  char * getId();
  char * getValueUnits();

  void setMetadata(char * strNewId, char * strNewTitle);
  void metadataFromAsciiGrid(char * strAsciiGrid);
  void metadataFromArcGrid(char * strArcGrid);
  void metadataFromEnvLayer(EnvLayer& fromLayer);

  void asSpecies();

  void toBitmap(char * strBitmap);
  void fromAsciiGrid(char * strAsciiGrid);
  void toAsciiGrid(char * strAsciiGrid);
  void fromArcGrid(char * strArcGrid);
  void toArcGrid(char * strArcGrid);
  void fromDataPoints(char * strDataPoints); //deprecated
  void fromDataPointString(char * strDataPoints);
  void fromDataPointStringSettingCounters(char * strDataPoints, EnvLayer * objMask,
      long& lMaskedPoints, long& lUnmaskedPoints, long& lUniqueMaskedPoints, long& lUniqueUnmaskedPoints);
  void fromEnvLayer(EnvLayer& fromLayer);

  void copyFromEnvLayer(EnvLayer& fromLayer);

  bool isCompatible(EnvLayer * objAnotherEnvLayer);

  void useBitmapMatrix(int iCDepth, int iColumns, int iRows);
  void useArcGridByteMatrix(char * strFilenameAux, double dCellSizeAux,
                double dXMaxAux, double dYMaxAux,
                double dXMinAux, double dYMinAux,
                double dMinValueAux, double dMaxValueAux);

  void createMatrix(char * strMatrixType, char * strMatrixFilename);
  ByteMatrix * getMatrix();
  void openMatrix();
  BYTE * getMatrixBytes(int &iSize);

#ifdef __GARPWIN
  void transferBytesToBitmap(BYTE * pBytes, int iColorDepth);
  void transferBytesToDIB(BYTE * pBytes);
#endif //__GARPWIN

};

// ==================================================================================
#endif //__ENVLAYER_H_
