// EnvLayerSet.h : Declaration of the CEnvLayerSet

//

// 11/19/03 Rich Williams - modified for use in SEEK

//



#ifndef __ENVLAYERSET_H_

#define __ENVLAYERSET_H_



//#include "resource.h"       // main symbols

//#include "Garp.h"

#include "Utilities.h"

#include "EnvLayer.h"

#include "EnvCell.h"

#include "EnvCellSet.h"



#include <scew.h>



// ====================================================================================

class EnvLayerSet

{

private:

  friend class CEnvLayerSet;



  // private properties

  char strId[CHAR_ARRAY_SIZE];

  char strTitle[CHAR_ARRAY_SIZE];

  char strFilename[CHAR_ARRAY_SIZE];

  char strDescription[16000];



  double dblXMin, dblXMax, dblYMin, dblYMax;

  double dblCellSize;

  int intRows, intColumns, intCells;



  char strMapUnits[CHAR_ARRAY_SIZE];

  char strCoordSys[CHAR_ARRAY_SIZE];



  // exception handling

  int intLastErrorCode;

  char strLastErrorMessage[CHAR_ARRAY_SIZE];



  // collection properties

  int intCount;

  EnvLayer * objEnvLayers[MAX_ENV_LAYERS];

  EnvLayer * objMaskLayer;

  EnvLayer * objSpeciesLayer;



  // aux array that stores the positions of all presence points

  // this is used by presample methods

  long intPresencePointPosition[MAX_PRESENCE_POINTS];

  int intPresencePointCount;

  bool blnPresencePointCalculated;



  int intLoadStatus;



public:

  // variables to count number of points

  long lMaskedPoints;

  long lUnmaskedPoints;

  long lUniqueMaskedPoints;

  long lUniqueUnmaskedPoints;





private:

  // private methods

  void getPositionOfPresencePoints();

  void filterMaskedPointsFromPresenceArray();



  void readXMLAttributes(scew_element const * pXMLSetNode);



public:

  EnvLayerSet();

  virtual ~EnvLayerSet();



  int size() const;

  int rows() const;

  int columns() const;



  void add(EnvLayer * layer);

  void addAsMask(EnvLayer * layer);

  void addAsSpecies(EnvLayer * layer);

  void swapLayers(int iFirstId, int iSecondId);

  void removeLayer(int index);

  EnvLayer * getLayerByIndex(int intIndex);

  EnvLayer * getLayerById(char * strId);

  EnvLayer * getMask();

  EnvLayer * getSpecies();



  bool masked(int intPos);

  void presampleProportional(EnvCellSet * cellSet, int intResamples);

  void presampleEqual(EnvCellSet * cellSet, int intResamples);

  void presampleFromBackgroundSurface(EnvCellSet * cellSet, int intResamples, int iBgSurfaceIndex);

  void presampleFromDataPoints(EnvCellSet * cellSet, char * strDataPoints, bool blnEqual, int intResamples, int iBgSurfaceIndex);

  void transferDataPointsToCellSet(EnvCellSet * cellSet, char * strDataPoints);

  void getEnvCombinations(EnvCellSet * cellSet);



  bool getValue(int intPos, EnvCell * cell);

  EnvCell * getCell(int intPos);



  void metadataFromEnvLayer(const EnvLayer& layer);

  void metadataToEnvLayer(EnvLayer * objEnvLayer);



  void resetWithMaskBytes(EnvLayer * objEnvLayer, BYTE bytValue);



  void loadText(char * strFilename, int iLoadMode);

  void saveText(char * strFilename);



  void fromXMLFile(char * strFilename, int intLoadFlag);

  void fromXML(char * strXML, int intLoadFlag);

  void fromXML(scew_element const * pXMLSetNode, int intLoadFlag);

  void toXMLFile(char * strOutputFilename);

  char * toXML();



  bool isCompatible(const EnvLayer& objAnotherEnvLayer);

};



#endif //__ENVLAYERSET_H_

