/* JUnit tests for CollectionPortBuffer class.
 *
 * Copyright (c) 2005 Natural Diversity Discovery Project.
 * All rights reserved.
 * 
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 * 
 * IN NO EVENT SHALL THE NATURAL DIVERSITY DISCOVERY PROJECT BE LIABLE 
 * TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
 * CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE AND 
 * ITS DOCUMENTATION, EVEN IF THE NATURAL DIVERSITY DISCOVERY PROJECT
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE NATURAL DIVERSITY DISCOVERY PROJECT SPECIFICALLY DISCLAIMS ANY 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE NATURAL 
 * DIVERSITY DISCOVERY PROJECT HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

   @ProposedRating Red (tmcphillips@naturaldiversity.org)
   @AcceptedRating Red (tmcphillips@naturaldiversity.org) 
 */
 
package org.nddp;

import ptolemy.data.DoubleToken;
import ptolemy.data.IntToken;
import ptolemy.data.StringToken;
import ptolemy.data.Token;
import junit.framework.TestCase;

/** 
 @author Timothy M. McPhillips
 */

public class TestCollectionPortBuffer extends TestCase {

	///////////////////////////////////////////////////////////////////
	////                         public methods                    //// 

    public void setUp() {
    	    
    		portBuffer = new CollectionPortBuffer();
    	    	intToken = new IntToken(0);
    	    	stringToken = new StringToken("string");
    	   	doubleToken = new DoubleToken(3.14159);
    	}


    	public void testFlushToPort() throws Exception {
    	    
    	    CollectionPortBuffer destinationPort = new CollectionPortBuffer();
    	    
    	    portBuffer.send(intToken);    	    
    	    portBuffer.send(stringToken);
    	    portBuffer.send(doubleToken);
    	    
    	    assertEquals(3, portBuffer.size());
    	    assertEquals(0, destinationPort.size());
    	
    	    portBuffer.flushToPort(destinationPort);

    	    assertEquals(0, portBuffer.size());
    	    assertEquals(3, destinationPort.size());

    	    assertEquals(intToken, destinationPort.get());
    	    assertEquals(stringToken, destinationPort.get());
    	    assertEquals(doubleToken, destinationPort.get());
    	}
    	
    	public void testFlushToPort_NullPort() throws Exception {
    	    
    	    portBuffer.send(stringToken);
    	    
    	    try {
    	        portBuffer.flushToPort(null);
    	        
    	        fail("A NullPointerException should have been thrown.");
    	    } 
    	    catch(NullPointerException expected) {
    	        assertEquals("Null collectionPort not allowed.",
                expected.getMessage());
    	    }
    	}
    	
    	public void testGet() throws Exception {
    	    
    	    portBuffer.send(intToken);    	    
    	    portBuffer.send(stringToken);
    	    portBuffer.send(doubleToken);
    	    
    	    assertEquals(intToken, portBuffer.get());
    	    assertEquals(2, portBuffer.size());

    	    assertEquals(stringToken, portBuffer.get());
    	    assertEquals(1, portBuffer.size());
    	    
    	    assertEquals(doubleToken, portBuffer.get());
    	    assertEquals(0, portBuffer.size());
    	}   

    	public void testGet_EmptyBuffer() throws Exception {

    	    assertEquals(null, portBuffer.get());
    	}
    	
    	public void testSend() throws Exception {
    	    
    	    assertEquals(0, portBuffer.size());
    	    
    	    portBuffer.send(intToken);
    	    assertEquals(1, portBuffer.size());
    	    
    	    portBuffer.send(stringToken);
    	    assertEquals(2, portBuffer.size());
    	    
    	    portBuffer.send(doubleToken);
    	    assertEquals(3, portBuffer.size());
    	}

    	
    	public void testSend_NullToken() throws Exception {
    	    
    	    try {
    	        portBuffer.send(null);
    	        
    	        fail("A NullPointerException should have been thrown.");
    	    } 
    	    catch(NullPointerException expected) {
    	        assertEquals("Null token not allowed.",
                expected.getMessage());
    	    }
    	}
    	
    ///////////////////////////////////////////////////////////////////
	////                         private variables                 ////

    	private Token doubleToken = new DoubleToken(3.14159);
    	private Token intToken = new IntToken(0);
    	private CollectionPortBuffer portBuffer;   	    
    	private Token stringToken = new StringToken("string");
}
