/**
 *  '$RCSfile: LSIDGeneratorTest.java,v $'
 *  Copyright: 2000 Regents of the University of California and the
 *              National Center for Ecological Analysis and Synthesis
 *    Authors: @authors@
 *    Release: @release@
 *
 *   '$Author: berkley $'
 *     '$Date: 2006/01/04 22:03:52 $'
 * '$Revision: 1.3 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package test.org.kepler.objectmanager.lsid;

import edu.ucsb.nceas.configxml.ConfigXML;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

import java.util.*;
import java.io.*;

import org.kepler.objectmanager.*;
import org.kepler.objectmanager.lsid.*;

import com.ibm.lsid.LSIDException;

public class LSIDGeneratorTest  extends TestCase
{
  
  public LSIDGeneratorTest(String name)
  {
    super(name);
    LSIDTree tree = LSIDTree.getInstance();
    tree.clearTree();
  }


  /**
   * Establish a testing framework by initializing appropriate objects
   */
  public void setUp()
  {
  }

  /**
   * Release any objects after tests are complete
   */
  public void tearDown()
  {
  }

  /**
   * Create a suite of tests to be run together
   */
  public static Test suite()
  {
    TestSuite suite = new TestSuite();
    suite.addTest(new LSIDGeneratorTest("initialize"));
    suite.addTest(new LSIDGeneratorTest("getNewLsidTest"));
    suite.addTest(new LSIDGeneratorTest("registerLSIDTest"));
    return suite;
  }

  /**
   * Run an initial test that always passes to check that the test
   * harness is working.
   */
  public void initialize()
  {
    assertTrue(1 == 1);
  }
  
  /**
   * test the newLsid method.
   */
  public void getNewLsidTest()
  {
    try
    {
      LSIDGenerator generator = new LSIDGenerator();
      generator.clearCache();
      KeplerLSID lsid1 = generator.getNewLSID("actor");
      assertEquals("kepler-project.org", lsid1.getAuthority());
      assertEquals("actor", lsid1.getNamespace());
      assertEquals("1", lsid1.getObject());
      assertEquals("1", lsid1.getRevision());
      
      KeplerLSID lsid2 = generator.getNewLSID("actor");
      assertEquals("kepler-project.org", lsid2.getAuthority());
      assertEquals("actor", lsid2.getNamespace());
      assertEquals("2", lsid2.getObject());
      assertEquals("1", lsid2.getRevision());
    }
    catch(Exception e)
    {
      fail("unexpected exception: " + e.getMessage());
    }
  }
  
  /**
   * test the registerLsid method.
   */
  public void registerLSIDTest()
  {
    try
    {
      LSIDGenerator generator = new LSIDGenerator();
      generator.clearCache();
      KeplerLSID lsid1 = new KeplerLSID("kepler-project.org", "actor", 1, 1);
      KeplerLSID lsid2 = new KeplerLSID("kepler-project.org", "actor", 2, 1);
      
      generator.registerLSID(lsid1);
      generator.registerLSID(lsid2);
      generator.registerLSID(lsid1);
      fail("should have thrown an exception because lsid1 already exists.");
    }
    catch(Exception e)
    {
      if(e instanceof LSIDException)
      {
        
      }
      else
      {
        fail("unexpected exception: " + e.getMessage());
      }
    }
  }
}