/**
 *    '$RCSfile: EcogridQueryClient.java,v $'
 *
 *     '$Author: ruland $'
 *       '$Date: 2006/01/17 20:39:50 $'
 *   '$Revision: 1.4 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.ecoinformatics.ecogrid.queryservice;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.Reader;
import java.net.URL;

import org.ecoinformatics.ecogrid.queryservice.util.EcogridQueryParser;
import org.ecoinformatics.ecogrid.queryservice.util.EcogridResultsetTransformer;

import org.ecoinformatics.ecogrid.queryservice.query.QueryType;
import org.ecoinformatics.ecogrid.queryservice.resultset.ResultsetType;
import org.ecoinformatics.ecogrid.queryservice.stub.EcoGridQueryInterfaceLevelOnePortType;
import org.ecoinformatics.ecogrid.queryservice.stub.EcoGridQueryLevelOneServiceLocator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author tao
 *
 * This client will connect to ecogrid query factory service and performed
 * query and get method
 */
public class EcogridQueryClient
{
   private EcoGridQueryInterfaceLevelOnePortType ecogrid;
   
   static Log log = LogFactory.getLog(EcogridQueryClient.class.getName());

   /**
    * Constructor create ecogrid client to connect given service url
    * @param serviceURL
    * @throws Exception
    */
   public EcogridQueryClient(String serviceURL) throws Exception
   {
      
      // Get a reference to the EcoGridQueryLevelOneService Factory
      long startFactory = System.currentTimeMillis();
      
      EcoGridQueryLevelOneServiceLocator loc = new
         EcoGridQueryLevelOneServiceLocator();
      
      ecogrid = loc.getQuery(new java.net.URL( serviceURL ));
      
      org.apache.axis.client.Stub stub = (org.apache.axis.client.Stub) ecogrid;
      
      stub.setTimeout(10000000);
      long endInstance = System.currentTimeMillis();
      
      log.debug("The time to create instance is =========== "+
                                (endInstance - startFactory)/1000);
      
   }
   
   /**
    * Query operation
    * @param query
    * @return
    * @throws Exception
    */
   public ResultsetType query(QueryType query) throws Exception
   {
      ResultsetType result = null;
      if (ecogrid != null)
         {
            long startQuery = System.currentTimeMillis();
            result = ecogrid.query(query); 
            long endQuery   = System.currentTimeMillis();
            log.debug("The query time is ==============" +
                                      (endQuery-startQuery)/1000);
         }
      return result;
      
   }
   
   
   
   /**
    * Method to query grid service base one query is a reader
    * @param queryText  the query text
    */
   public ResultsetType query(Reader queryText)
   {
      ResultsetType result = null;
      try
         {
            // create a parser
            EcogridQueryParser parser = new EcogridQueryParser(queryText);
            // get ecogrid query java object
            parser.parseXML();
            QueryType query = parser.getEcogridQuery();
            // check the query
            
            //System.out.println("view query");
            //viewQuery(query);
            
            // Call another query method
            result = query(query);
         }
      catch (Exception e)
         {
            System.err.println("ERROR!");
            e.printStackTrace();
         }
      return result;
   }//queryServer
   
   /**
    * Get operation
    * @param docid
    * @return
    */
   public byte[] get(String docid) throws Exception
   {
       byte[] data = null;
       if (ecogrid != null)
       {
          long startQuery = System.currentTimeMillis();
          data = ecogrid.get(docid); 
          long endQuery   = System.currentTimeMillis();
          log.debug("The query time is ==============" +
                                    (endQuery - startQuery)/1000);
       }
       return data;
   }
   
      
   public static void main(String[] args)
   {
      if (args.length != 3 && args.length != 4)
         {
            
            System.out.println("Usage: java  EcogridQueryClient <service url> <query|get> <query_filename|docid> (outputfile)");
            System.exit(0);
         }
      String GSH = args[0];
      String command = args[1];
      String commandarg = args[2];
      String outputfile = null;
      if ( args.length == 4 ) {
	      outputfile = args[3];
      }
       try
         {
            
            EcogridQueryClient client = new EcogridQueryClient( GSH );

	    String resultStr = null;

	    if ( "query".equalsIgnoreCase( command ) ) {
            
              File queryXMLFile = new File(commandarg);
              FileReader xmlReader = new FileReader(queryXMLFile);
            
              ResultsetType result = client.query(xmlReader);
            
              // print out the result set
              resultStr = EcogridResultsetTransformer.toXMLString(result);
              System.out.println(resultStr);

	    }
	    else if ( "get".equalsIgnoreCase( command ) ) {
              System.out.println("start to get docoument");
              byte[] data = client.get(commandarg);
	      resultStr = new String( data );
              System.out.println(resultStr);
              System.out.println("after get doc and testing is done");
	    }
	    else {
              System.out.println( "Unknown command: " + command );
	      return;
            }
           
	    if ( outputfile != null ) {
	      System.out.println( "Creating outputfile: " + outputfile );
	      File outFile = new File( outputfile );
	      FileWriter outWriter = new FileWriter( outFile );
	      PrintWriter w = new PrintWriter( outWriter );
	      w.println( resultStr );
	      w.flush();
	      w.close();
	    }
            
         }
      catch (Exception e)
         {
            System.err.println("Exception in main : " + e.getMessage());
            e.printStackTrace();
         }
      
   }//main
}
