/**
 *    '$RCSfile: Eml200DataSource.java,v $'
 *
 *     '$Author: tao $'
 *       '$Date: 2006/02/28 23:11:10 $'
 *   '$Revision: 1.100 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.seek.datasource.eml.eml2;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ecoinformatics.seek.dataquery.DBTableNameResolver;
import org.ecoinformatics.seek.dataquery.DBTablesGenerator;
import org.ecoinformatics.seek.dataquery.HsqlDataQueryAction;
import org.ecoinformatics.seek.datasource.DataSourceIcon;
import org.ecoinformatics.seek.datasource.EcogridDataCacheItem;
import org.ecoinformatics.seek.datasource.EcogridGZippedDataCacheItem;
import org.ecoinformatics.seek.datasource.EcogridMetaDataCacheItem;
import org.ecoinformatics.seek.datasource.EcogridTarArchivedDataCacheItem;
import org.ecoinformatics.seek.datasource.EcogridZippedDataCacheItem;
import org.ecoinformatics.seek.ecogrid.quicksearch.ResultRecord;
import org.ecoinformatics.seek.ecogrid.quicksearch.ResultTreeRoot;
import org.ecoinformatics.seek.querybuilder.DBQueryDef;
import org.ecoinformatics.seek.querybuilder.DBQueryDefParserEmitter;
import org.ecoinformatics.seek.querybuilder.DBSchemaParserEmitter;
import org.ecoinformatics.util.DelimitedReader;
import org.ecoinformatics.util.Util;
import org.kepler.objectmanager.cache.DataCacheListener;
import org.kepler.objectmanager.cache.DataCacheManager;
import org.kepler.objectmanager.cache.DataCacheObject;
import org.kepler.objectmanager.data.DataSourceControllerFactory;
import org.kepler.objectmanager.data.DataType;
import org.kepler.objectmanager.data.db.DSSchemaDef;
import org.kepler.objectmanager.data.db.DSSchemaIFace;
import org.kepler.objectmanager.data.db.DSTableFieldIFace;
import org.kepler.objectmanager.data.db.Entity;
import org.kepler.objectmanager.data.db.QBTableauFactory;
import org.kepler.objectmanager.data.text.TextComplexFormatDataReader;
import org.xml.sax.InputSource;

import ptolemy.actor.TypedIOPort;
import ptolemy.actor.gui.style.TextStyle;
import ptolemy.data.DoubleToken;
import ptolemy.data.IntToken;
import ptolemy.data.LongToken;
import ptolemy.data.StringToken;
import ptolemy.data.Token;
import ptolemy.data.expr.FileParameter;
import ptolemy.data.expr.StringParameter;
import ptolemy.data.type.BaseType;
import ptolemy.data.type.Type;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.ChangeRequest;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;
import ptolemy.kernel.util.NamedObj;
import ptolemy.kernel.util.Settable;
import ptolemy.kernel.util.StringAttribute;
import ptolemy.moml.MoMLChangeRequest;
import EDU.oswego.cs.dl.util.concurrent.CountDown;
import EDU.oswego.cs.dl.util.concurrent.Latch;


/**
 * A ResultRecord that is in EML 2.0.0 format.
 * 
 * Brief discussion of threads:
 * 
 * The actor startup will call attributeChanged multiple time to configure the object.
 * When the recordId and endpoint attributes are set, the object will attempt to load
 * the EcoGridMetaDataCacheItem.  This operation will fork a thread named "MetaData XYZZY".
 * 
 * The MetaData thread is configured to use the inner class MetadataComplete listener
 * when it completes.  The thread, in this listener, initializes the CountDown _entityCountDown
 * and kicks off threads to load each entity.  The thread then waits using _entityCountDown.acquire()
 * until all the data entity thread are complete.
 * 
 *  Each data entity is loaded using a class derived from EcogridDataCacheItem.  The specific type
 *  of class is determined from 
 */
public class Eml200DataSource extends ResultRecord implements DataCacheListener
{
	
	static final String DATATYPES[] = { DataType.INT, DataType.FLOAT,
		DataType.DOUBLE, DataType.LONG, DataType.STR};
	
	static final BaseType BASETYPES[] = { BaseType.INT, BaseType.DOUBLE,
		BaseType.DOUBLE, BaseType.LONG, BaseType.STRING};
	
	static private Hashtable _TypeHash = new Hashtable();
	
	static Log log;
	static {
		log = LogFactory.getLog("org.ecoinformatics.seek.datasource.eml.eml2.Eml200DataSource");
	}
	
	private QBTableauFactory _qbTableauFactory = null;
	
	private DataSourceControllerFactory _nodeController = null;
	
	///////////////////////////////////////////////////////////////////
	//// private variables ////
	
	/**
	 * Output indicator parameter.
	 */
	private Eml200OutputTypeBase _outputType = null;
	
	private Vector _entityList  = new Vector();
	private Entity _selectedTableEntity = null;
	
	private Vector _columns = null;
	
	private DBQueryDef _queryDef = null;
	
	private DSSchemaIFace _schemaDef = null;
	private DSSchemaDef _schema       = new DSSchemaDef();
	
	private boolean _ignoreSchemaChange = false;
	
	private boolean _schemaAttrIsSet = false;
	
	private DBTablesGenerator _tableGenerator = null;
	
	private EcogridDataCacheItem _selectedCachedDataItem = null;
	
	private EcogridMetaDataCacheItem _cacheMetaDataItem = null;
 
    /**
     * _metadataCompleted is a Latch object (from oswego) which is used
     * to synchronize the start of the workflow.  The Latch is released
     * when the _cacheMetaDataItem has been completed.  It is also used to
     * notify a thread blocked in initialized() when the stop method
     * is executed.
     */
    private Latch                  _metadataCompleted = new Latch();
	
	private InputStream _reader = null;
	
	private HsqlDataQueryAction _resultSet = null;
	
	/*
	 * Indicates when there is no more data in the _resultSet to output.
	 */
	private boolean _endOfResultSet = false;
	
	private DelimitedReader _simpleDelimitedReader = null;
	
	private TextComplexFormatDataReader _complexFormatReader = null;
	
	private CountDown _entityCountDown = null;
	private int _numberOfEntities = 0;
	
	private int _numberOfFailedDownloadEntities = 0;
	
	private boolean _hasSQLCommand        = false;
	
	private String[] _selectedColumnLabelList = null;
	
	private Type[] _selectedColumnTypeList    = null;
	
	private static final int   INDEXFORNOTFOUND        = -1;
	
	private static final int   DEFAULTINDEX            = 0;
	
	private static final String _SELECTEDENTITY        = "Selected Entity";
	
    private DataSourceIcon         _icon;

    ///////////////////////////////////////////////////////////////////
	//// ports and parameters ////
	
	private FileParameter _datasetPathAttr = null;
	
	private StringAttribute _sqlAttr = null;
	
	private StringAttribute _schemaAttr = null;
	
	private StringParameter _outputTypeAttr = null;
	
	private StringParameter _fileExtensionInZip = null;
	
	private StringParameter _selectedEntityAttr    = null;
	
	/**
	 * Construct an actor with the given container and name.
	 * 
	 * @param container
	 *            The container.
	 * @param name
	 *            The name of this actor.
	 * @exception IllegalActionException
	 *                If the actor cannot be contained by the proposed
	 *                container.
	 * @exception NameDuplicationException
	 *                If the container already has an actor with this name.
	 * @since
	 */
	public Eml200DataSource(CompositeEntity container, String name)
	throws NameDuplicationException, IllegalActionException
	{
		super(container, name);
        _icon = new DataSourceIcon( this );

        _datasetPathAttr = new FileParameter(this, "EML File");
		
		_schemaAttr = new StringAttribute(this, "schemaDef");
		TextStyle schemaDefTS = new TextStyle(_schemaAttr, "schemaDef");
		
		_sqlAttr = new StringAttribute(this, "sqlDef");
		TextStyle sqlDefTS = new TextStyle(_sqlAttr, "sqlDef");
		_selectedEntityAttr = new StringParameter(this, _SELECTEDENTITY);
		_outputTypeAttr = new StringParameter(this, "outputType");
		_outputTypeAttr.setExpression(Eml200OutputTypeFactory._AsField);
		_outputTypeAttr.addChoice(Eml200OutputTypeFactory._AsField);
		_outputTypeAttr.addChoice(Eml200OutputTypeFactory._AsTable);
		_outputTypeAttr.addChoice(Eml200OutputTypeFactory._AsRow);
		_outputTypeAttr.addChoice(Eml200OutputTypeFactory._AsByteArray);
		_outputTypeAttr.addChoice(Eml200OutputTypeFactory._AsUnzippedFileName);
		_outputTypeAttr.addChoice(Eml200OutputTypeFactory._AsFileName);
		_outputTypeAttr.addChoice(Eml200OutputTypeFactory._AsColumnVector);
		_outputTypeAttr.addChoice(Eml200OutputTypeFactory._AsColumnRecord);
		//_outputType = Eml200OutputTypeFactory.newInstance(Eml200OutputTypeFactory._AsField,this);
		_outputType = Eml200OutputTypeFactory.newInstance(this);
		
		_fileExtensionInZip = new StringParameter(this, "Target File Extension in Compressed File");
		
		// create tableau for editting the SQL String
		_qbTableauFactory = new QBTableauFactory(this, "_tableauFactory");
		
		// Create a node controller to control the context menu
		_nodeController = new DataSourceControllerFactory(this, 
		"_controllerFactory");
		
		if (_TypeHash.size() == 0) {
			for (int i = 0; i < DATATYPES.length; i++) {
				_TypeHash.put(DATATYPES[i], BASETYPES[i]);
			}
		}
	}

	/**
	 * Accessor to _columns member.  Default permissions for use by Eml200OutputTypeBase derived
	 * classes only.
	 * @return
	 */
	Vector getColumns() {
		return _columns;
	}

    /**
	 * @return Returns the _selectedTableEntity.
	 */
	Entity getSelectedTableEntity() {
		return _selectedTableEntity;
	}

	/**
	 * @return Returns the _selectedCachedDataItem.
	 */
	EcogridDataCacheItem getSelectedCachedDataItem() {
		return _selectedCachedDataItem;
	}

	/**
	 * @return Returns the _fileExtensionInZip.
	 */
	String getFileExtensionInZip() {
		try {
			return _fileExtensionInZip.stringValue();
		}
		catch (IllegalActionException e) {
			return "";
		}
	}

	/**
	 * @return Returns the _selectedColumnLabelList.
	 */
	String[] getColumnLabels() {
		return _selectedColumnLabelList;
	}

	/**
	 * @return Returns the _selectedColumnTypeList.
	 */
	Type[] getColumnTypes() {
		return _selectedColumnTypeList;
	}

	public void preinitialize() throws IllegalActionException {

		// First block for metadata download to finish.
        try {
            _metadataCompleted.acquire();
            log.debug("Is stop requested? " + getDirector().isStopRequested());
        }
        catch( InterruptedException e ) {
            log.debug("Is stop requested? " + getDirector().isStopRequested());
            if ( getDirector().isStopRequested() ) {
                throw new IllegalActionException("Execution interrupted");
            }
        }

        super.preinitialize();
	}

	/**
	 * Initialize the actor prior to running in the workflow. This reads the
	 * metadata and configures the ports.
	 * 
	 * @throws IllegalActionException
	 */
	public void initialize() throws IllegalActionException
	{
		log.debug("In initialize method");

        // Now block waiting for the entity data to finish.
        try {
            synchronized( _entityCountDown ) {
                while ( _entityCountDown.currentCount() > 0 && !getDirector().isStopRequested() ) {
                    _entityCountDown.wait();
                    log.debug("Is stop requested? " + getDirector().isStopRequested());
                }
            }
        }
        catch( InterruptedException e ) {
            throw new IllegalActionException("Downloads not completed");
        }

        if ( getDirector().isStopRequested() ) {
            throw new IllegalActionException("Execution interrupted");
        }
        
		if (_selectedTableEntity == null) 
		{
			throw new IllegalActionException("_selectedTableEnity is NULL!");
		}
		
		if (_selectedCachedDataItem == null)
		{
			_selectedCachedDataItem = (EcogridDataCacheItem) 
			_selectedTableEntity.getDataCacheObject();
		}
		
		if (_selectedCachedDataItem == null)
		{
			throw new IllegalActionException(" The selected entity has a null data (Maybe data download failed)");
		}

		// This was the initializeAsTableRowOrField method
		String sqlStr = "";
		String sqlXMLStr = ( (Settable) _sqlAttr).getExpression();
		if (sqlXMLStr == null || sqlXMLStr.length() == 0) {
			sqlStr = "SELECT * FROM "
				+ (_selectedTableEntity.getMappedName() != null ?
						_selectedTableEntity.getMappedName() :
						_selectedTableEntity.getName());
			
		}
		else {
			Hashtable mappedNameHash = new Hashtable();
			// should go through all enities
			int size = _entityList.size();
			for (int i=0; i< size; i++)
			{
				Entity entity = (Entity)_entityList.elementAt(i);
				if (entity.getMappedName() != null)
				{
					mappedNameHash.put(entity.getName(), entity.getMappedName());
				}
			}
			DBQueryDef queryDef = DBQueryDefParserEmitter.parseQueryDef(
					               _schemaDef, sqlXMLStr, mappedNameHash);
			
			sqlStr = DBQueryDefParserEmitter.createSQL(_schemaDef, queryDef);
		}
		
		log.debug("The sql command is "+sqlStr);
		// excuted query
		if (sqlStr != null && !sqlStr.trim().equals("")) {
			// if table gnerated successfully, we will run query
			if (_tableGenerator != null && _tableGenerator.getSuccessStatus()) {
				try {
                    _icon.setBusy();
					_resultSet = new HsqlDataQueryAction();
					_resultSet.setSQL(sqlStr);
					_resultSet.actionPerformed(null);
					
				}
				catch (Exception e) 
				{
					log.debug("Error to run query is ", e );
					throw new IllegalActionException(e.getMessage());
				}
				
			}
		}
		//if reustlset is null(this can be caused by db couldn't create table)
		// and we don't have any sql command
		//(this means we only have one data entity involve) or only has one entity at all
		// we would like to try read the selected entity data from datachache rather than
		// from db
		if (_resultSet == null &&  (!_hasSQLCommand || _numberOfEntities==1))
		{
			//System.out.println("in result set is null!!!!!!!!!!!!!!!!!!");
			_reader = _selectedCachedDataItem.getDataInputStream();
			try
			{
				createDelimitedReader();
			}
			catch (Exception e) 
			{
				log.debug("Error to run delimiter reader is  ", e);
				throw new IllegalActionException(e.getMessage());
			}
			
		}
        _icon.setReady();
		// This was the initializeAsTableRowOrField method

        // Set marker to say we have data.  This might not be true though and is perhaps
        // a bug.  The correct thing to do, is check for data in the prefire and in the
        // postfire.
        _endOfResultSet = false;
        
        _outputType.initialize();
		
	}
	
	
	
	/*
	 * This method will reader a row vector from data source, either from
	 * resultset which excuted by data query or delimiterdReader which reader
	 * from data inputtream - _reader. This method will be called in asFired**
	 * method
	 */
	Vector gotRowVectorFromSource() throws Exception
	{
		Vector rowVector = new Vector();
		if (_resultSet != null)
		{
			
			ResultSet rs = _resultSet.getResultSet();
			ResultSetMetaData metadata = rs.getMetaData();
			int columnSize = metadata.getColumnCount();
			
			if(rs.next()) 
			{
				for (int i = 0; i < columnSize; i++) 
				{
					String str = rs.getString(i + 1);
					rowVector.add(str);
				}
			}
		}
		else if (_reader != null && (!_hasSQLCommand || _numberOfEntities==1))
		{
			if (_selectedTableEntity.isSimpleDelimited())
			{
				_simpleDelimitedReader.setCollapseDelimiter(_selectedTableEntity.getCollapseDelimiter());
				_simpleDelimitedReader.setNumFooterLines(_selectedTableEntity.getNumFooterLines());
				rowVector = _simpleDelimitedReader.getRowDataVectorFromStream();
			}
			else
			{
				rowVector = _complexFormatReader.getRowDataVectorFromStream();
			}
			
		}
		
		if (rowVector.isEmpty() ) {
			_endOfResultSet = true;
		}
		return rowVector;
	}
	
	
	/* (non-Javadoc)
	 * @see ptolemy.actor.lib.Source#prefire()
	 */
	public boolean prefire() throws IllegalActionException {
		return _outputType.prefire();
	}

	/**
	 * Send a record's tokens over the ports on each fire event.
	 * 
	 * @exception IllegalActionException
	 *                If there is no director.
	 */
	public void fire() throws IllegalActionException
	{
		//log.debug("In fire method");
		super.fire();
		_outputType.fire();
	}
	
	/** 
	 * This method is only for output as byte array. 
	 * Read the next bytes from the input stream into array. Fire method will send
	 *  the array out. If there reached EOF ,
	 *  return false.  Otherwise, return whatever the superclass returns.
	 *  @exception IllegalActionException If there is a problem reading
	 *   the file.
	 */
	public boolean postfire() throws IllegalActionException {
		
		if ( !_outputType.postfire() ) {
			return false;
		}
		
		try {
			if ( _resultSet != null && _resultSet.getResultSet().isAfterLast() ) {
				return false;
			}
		}
		catch( SQLException e ) {
			throw new IllegalActionException(this,e,"Unable to determine end of result set");
		}
		
		if ( _endOfResultSet ) {
			return false;
		}
		
		return super.postfire();
		
	}
	
	void initializePort( String aPortName, Type aPortType )  throws IllegalActionException
	{
		try {
			String columnName = aPortName.trim();
			// Create a new port for each Column in the resultset
			TypedIOPort port = (TypedIOPort) this.getPort(columnName);
			boolean aIsNew = (port == null);
			if (aIsNew) {
				// Create a new typed port and add it to this container
				port = new TypedIOPort(this, columnName, false, true);
				log.debug("Creating port [" + columnName + "]" + this);
			}
			port.setTypeEquals( aPortType );
			
		} catch (ptolemy.kernel.util.NameDuplicationException nde) {
			throw new IllegalActionException("One or more attributes has the same name.  Please correct this and try again.");
		}
		
	}
	
	/*
	 * Remove all ports which's name is not in the selected vector
	 */
	void removeOtherOutputPorts(Collection nonRemovePortName) throws IllegalActionException
	{
		// Use toArray() to make a deep copy of this.portList().
		// Do this to prevent ConcurrentModificationExceptions.
		TypedIOPort[] l = new TypedIOPort[0];
		l = (TypedIOPort[])this.portList().toArray(l);

		for (int i = 0; i<l.length; i++) {
			TypedIOPort port = l[i];
			if ( port == null || port.isInput() ) {
				continue;
			}
			String currPortName = port.getName();
			if (!nonRemovePortName.contains(currPortName))
			{
				try {
					port.setContainer(null);
				}
				catch (Exception ex) {
					throw new IllegalActionException(this,
							"Error removing port: " + currPortName);
				}
			}
		}
	}
	

    /**
     * Issue a ChangeRequest to change the output ports.
     * @throws ptolemy.kernel.util.IllegalActionException
     */
    private void reconfigurePorts(String why) {
        log.debug("Creating reconfigure ports change request " + why);
        this.requestChange( new ChangeRequest(this,why ) {
            public void _execute() throws Exception {
                log.debug("Executing reconfigure ports change request " + this.getDescription());
                _outputType.reconfigurePorts();
            }
        });
    }
	
	/**
	 * Callback for changes in attribute values.
	 */
	public void attributeChanged(ptolemy.kernel.util.Attribute attribute)
	throws ptolemy.kernel.util.IllegalActionException
	{
		//log.debug("In attribute change method");
		if (attribute == _sqlAttr) {
			String sqlDef = ((Settable) attribute).getDefaultExpression();
			sqlDef = ((Settable) attribute).getExpression();
			if (sqlDef.length() > 0) {
				_hasSQLCommand = true;
				_queryDef = DBQueryDefParserEmitter.parseQueryDef(_schemaDef,
						sqlDef);
				_columns = _queryDef.getSelects();
                reconfigurePorts("Sql attribute changed");
			}
			
		} else if (attribute == _schemaAttr && !_ignoreSchemaChange) 
			// NOTE: We may skip setting it here because _ignoreSchemaChange
			// may be true
		{
			String schemaDef = ((Settable) _schemaAttr).getExpression();
			
			// MOML may have a blank definition
			if (schemaDef.length() > 0) {
				_schemaAttrIsSet = true; // remember that we have been set by
				// the MOML
				
				log.debug("schemaDef >>" + schemaDef + "<<");
				
				_schemaDef = DBSchemaParserEmitter.parseSchemaDef(schemaDef);
			}
			
		} else if ((attribute.getName().equals("recordId") ||
				    attribute.getName().equals("endpoint"))
				&& this.hasConnectionValues()
				&& !(attribute.getContainer().getContainer() instanceof ResultTreeRoot)) {
			log.debug("=========================change recorid or endpoints");
            _icon.setBusy();
			_cacheMetaDataItem = (EcogridMetaDataCacheItem) DataCacheManager
			.getCacheItem(new MetadataComplete(),
					"MetaData " + getRecordId(), getEndpoint(),
					EcogridMetaDataCacheItem.class.getName());
            if ( _cacheMetaDataItem.isEmpty() ) {
                _cacheMetaDataItem.setEndPoint(getEndpoint());
                _cacheMetaDataItem.setRecordId(getRecordId());
                _cacheMetaDataItem.start();
            }
			
			
		} else if (attribute == _outputTypeAttr) {
			String outputType = ((Settable) attribute).getExpression();
			log.debug("=========================change outputtype"+outputType);
			String strOutputType = _outputTypeAttr.stringValue();
			_outputType = Eml200OutputTypeFactory.newInstance( strOutputType, this);
            reconfigurePorts("Output type changed");

		}else if (attribute == _selectedEntityAttr){
			//reset selected entity
			String selectedEntityName = ((Settable) attribute).getExpression();
			log.debug("=========================selected entity "+selectedEntityName);
			setSelectedEntityValue(true);
			
			
		}
		
		
	}
	
	/**
	 * Creates the schema definition from the incoming data columns
	 */
	private void createSchemaFromData(Entity tableEntity)
	{
		try // XXX shouldn't be catch this exception here
		{
			
			// _schemaAttrIsSet gets set when the schema has come from the MOML
			// So if it is false here, then we set the attribute from the data.
			//
			// If after this the attr gets set from the MOML it will override
			// what we set here.
			//Entity tableEntity = dataItem.getEntity();
			DataCacheObject dataItem = tableEntity.getDataCacheObject();
			if (tableEntity != null) {
				_schema.addTable(tableEntity);
				_schemaDef = _schema;
				
				DBTableNameResolver nameResolver = new DBTableNameResolver();
				try {
					tableEntity = nameResolver.resolveTableName(tableEntity);
				} catch (Exception e) {
				}
				boolean refresh = false;
				if (tableEntity.isSimpleDelimited())
				{
					// for simple dimilter, we can use text table
					_tableGenerator = 
						new DBTablesGenerator(tableEntity, dataItem.getBaseFileName(),refresh); 
				}
				else
				{
					// for complex format, we should use inpustream to create table
					InputStream input = dataItem.getDataInputStream();
					_tableGenerator = new DBTablesGenerator(tableEntity, input, refresh);
				}
				
				_tableGenerator.run(); // don't do thread
				
				String schemaDefXML = DBSchemaParserEmitter.emitXML(_schemaDef);
				_ignoreSchemaChange = true;
				((Settable) _schemaAttr).setExpression(schemaDefXML);
				_ignoreSchemaChange = false;
			}
		} catch (IllegalActionException e) {
			log.debug("In createSchemaFromData: " + e);
		}
		
	}
	
	
	/**
	 * Get a URL pointer to the documentation for this data source. The XML 
	 * source of the EML document is retrieved from the cache, and then passed 
	 * to an XSLT parser to be transformed into HTML format, which is saved in 
	 * a temporary file. The URL of the temporary file containing the HTML 
	 * result is returned.
	 * 
	 * @return URL the URL of the HTML file containing the documentation
	 */
	public URL getDocumentation()
	{
		String htmlFileName = getRecordId();
		String namespace = getNamespace();
		if (namespace == null)
		{
			namespace = EML2MetadataSpecification.EML200NAMESPACE;
		}
		log.debug("The name space is "+namespace);
		URL htmlUrl = null;
		// Get the metadata XML document and transform it to html
		if (_cacheMetaDataItem.isReady()) {
			try {
				InputStream is = _cacheMetaDataItem.getDataInputStream();
				InputStreamReader source = new InputStreamReader(is);
				htmlUrl = Util.getMetadataHTMLurl(source, namespace, htmlFileName);
				
			} catch (Exception fnfe) {
				log.debug("Could not open temporary output file.");
			}
		}
		return htmlUrl;
	}
	
	
	
	/**
	 * Creates the delimtedReader from the data item inputstream
	 */
	private void createDelimitedReader() throws Exception
	{
		/*String data = aDataStr.toString();
		 log.debug("-----------------\n" + data
		 + "\n-----------------\n", 2);*/
		
		//log.debug("entityhash: " + entityhash.toString());
		if (_selectedTableEntity.isSimpleDelimited())
		{
			boolean stripHeaderLine = true;
			_simpleDelimitedReader = new DelimitedReader(_reader, _selectedTableEntity
					.getAttributes().length, _selectedTableEntity.getDelimiter(),
					_selectedTableEntity.getNumHeaderLines(), _selectedTableEntity.getRecordDelimiter(), 
					_selectedTableEntity.getNumRecords(), stripHeaderLine);
		}
		else
		{
			_complexFormatReader = new TextComplexFormatDataReader(_reader, _selectedTableEntity);
		}
		
		//_dataVectors = dr.getTokenizedData(true);
	}
	
	/**
	 * Returns a Ptolemly type for a given Kepler DataSource Type
	 * @param aType DataSource type
	 * @return Ptolemy type
	 */
	BaseType getBaseType(String aType)
	{
		BaseType type = (BaseType) _TypeHash.get(aType);
		if (type == null) { return BaseType.UNKNOWN; }
		return type;
	}
	
	/**
	 * Inner class used for completion notification of EcoGridMetaDataCacheItem objects.
	 */
	private class MetadataComplete implements DataCacheListener {
		
		public void complete(DataCacheObject aItem) {
			log.debug("MetadataComplete: " + this);

            try {
                aItem.removeListener(this);
			
                if (!aItem.isReady()) 
                {
                    log.error("Unable to download MetaData");
                    _icon.setError();
                    return;
                }
			
                try {
                    parsePackage(new InputStreamReader(new FileInputStream(new File(aItem.getAbsoluteFileName()))));
                } catch (Exception e) {
                    log.error("Exception occurred during MetaDataCompletion",e);
                    _icon.setError();
                }
    			// if no sql command, we need set cloumns
    			if (!_hasSQLCommand)
    			{
    				log.debug("There is no sql command attribute and set up cloumns in compelete method");
    				_columns = _selectedTableEntity.getFields();
    			}

    			generteLabelListAndTypteListFromColumns();
    			reconfigurePorts("Metadata Complete");

            }
            finally {
                _metadataCompleted.release();
            }

		}
	}
	
	//------------------------------------------------------------------------
	//-- DataCacheListener
	//------------------------------------------------------------------------
	
	
	public void complete(DataCacheObject aItem)
	{
		log.debug("complete: " + this);
		
		log.debug("Class of aItem " + aItem.getClass().getName() );
		aItem.removeListener(this);
		if (aItem.isReady()) 
		{
			log.debug("aItem is instanceof EcogridDataCacheItem");
			
			EcogridDataCacheItem item = (EcogridDataCacheItem)aItem;
			String entityIdentifier = item.getEntityIdentifier();
			int index = lookupEntityListName(entityIdentifier);
			Entity entity = null;
			if (index != INDEXFORNOTFOUND)
			{
				entity = (Entity) _entityList.elementAt(index);
			}
			entity.setDataCacheObject(item);    
			if (!entity.getIsImageEntity()) {
				
				createSchemaFromData(entity);
				
			}
		} else if (aItem.isError()) {
			log.debug("In failed download path");
			EcogridDataCacheItem item = (EcogridDataCacheItem)aItem;
			String entityIdentifier = item.getEntityIdentifier();
			int index = lookupEntityListName(entityIdentifier);
			Entity entity = null;
			if (index != INDEXFORNOTFOUND)
			{
				entity = (Entity) _entityList.elementAt(index);
			}
			//because fail to download, set null as data cache item
			entity.setDataCacheObject(null);
			_numberOfFailedDownloadEntities++;
		}
		// Decrement the number of downloads completed.
		_entityCountDown.release();
        
        // Check for completion
        finished();
	}
	
	
	
	/*
	 * parses the package and fills in the names and types arrays
	 */
	private void parsePackage(Reader eml) throws IllegalActionException
	{
		Eml200Parser eml2parser;
		
		try { //parse the package for the names and types of the atts
			log.debug("creating parser");
			eml2parser = new Eml200Parser();
			log.debug("parsing...");
			eml2parser.parse(new InputSource(eml));
			// get if this package has image entity
			//_hasImageEntity = eml2parser.getHasImageEntity();
			//log.debug("This pakcage has image entity " + _hasImageEntity);
			log.debug("Done parsing");
		} catch (Exception e) {
			throw new IllegalActionException("Error parsing the eml package: "
					+ e.getMessage());
		}
		
		getData(eml2parser); // fills in the _dataVectors data member
		setOptionsForSelectedEntityAttribute();
	
        finished();
    }

    private void finished() {
        
        if ( _entityCountDown.currentCount() > 0 ) {
            return;
        }
		
		try
		{
			// this method will set up cloumns info if not sql command
			setSelectedEntityValue(false);
			
		}
		catch(Exception e)
		{
			log.debug("The error in set up selected entity is ", e);
			
		}
		if ( _numberOfFailedDownloadEntities == 0 ) 
		{
            _icon.setReady() ;
		}
		else 
		{
			log.error("Some downloads failed");
            _icon.setError();
		}
  
	}
	
	/**
	 * get the data based on the contents of the package
	 * 
	 * @param parser
	 *            the parser that has already parsed the package.
	 */
	private void getData(Eml200Parser parser)
	throws IllegalActionException
	{
		if (parser != null)
		{
			_numberOfEntities = parser.getEntityCount();
			_entityCountDown = new CountDown( _numberOfEntities );
			/*if (parser.getEntityCount() > 1) {
			 throw new IllegalActionException(
			 "Currently this parser only deals with one entity. "
			 + "Please use a package with only one entity.");*/
			if ( _numberOfEntities == 0) 
			{
				throw new IllegalActionException(
				"There must be at least one entity in the EML package.");
			}
			Hashtable entityList = parser.getEntityHash();
			// initialize selected entity
			_selectedTableEntity = (Entity) entityList.get(entityList.keys().nextElement());
			//log.debug("entityhash: " + _entityList.toString());
			
			// start a thread to get data chachedItem
			Enumeration enu = entityList.elements();
			while(enu.hasMoreElements())
			{
				
				Entity singleEntity = (Entity)enu.nextElement();
				log.debug("Adding Entity " + singleEntity);
				_entityList.add(singleEntity);
				getDataItemFromCache(singleEntity);
			}
			
			
			
		}
	}
	
	/*
	 * This method will start a thread to get chached data item base on the url
	 * getting from table entity. This method will be call in method getdatac and
	 * wrapup
	 */
	private void getDataItemFromCache(Entity tableEntity)
	{
		if (tableEntity == null)
		{
			log.debug("The table enity is null and couldn't get chached data item");
			return;
		}
		String fileURLStr = tableEntity.getURL();
		log.debug("Data URL is: " + fileURLStr);
		// we need to distinguish zip file and generate
		//String compression = tableEntity.getCompressionMethod();
		EcogridDataCacheItem cachedDataItem = null;
		if (tableEntity.getHasZipDataFile())
		{
			log.debug("This is a zip data cacheItem");
			cachedDataItem = (EcogridZippedDataCacheItem) 
			DataCacheManager.getCacheItem(this, "Data " + fileURLStr, fileURLStr, EcogridZippedDataCacheItem.class.getName());
			//_isZipDataFile = true;
		}
		else if (tableEntity.getHasGZipDataFile())
		{
			log.debug("This is a gzip data cacheItem");
			cachedDataItem = (EcogridGZippedDataCacheItem) 
			DataCacheManager.getCacheItem(this, "Data " + fileURLStr, fileURLStr, EcogridGZippedDataCacheItem.class.getName());
			if (tableEntity.getHasTarDataFile())
			{
				log.debug("This is gizp and tar data cache item");
				cachedDataItem.setIsTarFile(true);
			}
			
		}
		else if (tableEntity.getHasTarDataFile())
		{
			log.debug("This is a tar data cacheItem");
			cachedDataItem = (EcogridTarArchivedDataCacheItem) DataCacheManager.getCacheItem(this, "Data " + fileURLStr, fileURLStr, EcogridTarArchivedDataCacheItem.class.getName());
		}
		else
		{
			log.debug("This is a uncompressed data cacheItem");
			cachedDataItem = (EcogridDataCacheItem) DataCacheManager.getCacheItem(this, "Data " + fileURLStr, fileURLStr, EcogridDataCacheItem.class.getName());
		}
        if ( cachedDataItem.isEmpty() ) {
            cachedDataItem.setEndPoint(this.getEndpoint());
            // add entity identifier order to track the cachedata item associate with 
            // which entity(In complete method)
            cachedDataItem.setEntityIdentifier(tableEntity.getName());
            cachedDataItem.start();
        }
		
	}
	
	/*
	 * This method will set options for "Selected Entity Attribute" after get all
	 * data entities. The options will be the entities' name
	 */
	private void setOptionsForSelectedEntityAttribute()
	{
		if (_entityList != null)
		{
			int length = _entityList.size();
			for (int i=0;  i<length; i++)
			{
				Entity entity = (Entity)_entityList.elementAt(i);
				String entityName = entity.getName();
				if (entityName != null && !entityName.trim().equals(""))
				{
					_selectedEntityAttr.addChoice(entityName);
				}
				
			}
			
		}
	}
	
	/*
	 * Method to set up selected entity. If Attribute "selectedEntity" already
	 * has value, look up the enitytList and found the selected entity. If 
	 * attribute "selectedEntity doesn't have any value, we choose the index 0
	 * as selected entity.
	 */
	private void setSelectedEntityValue(boolean fromAttributeChange) throws IllegalActionException
	{
		
		String selectedEntityName = _selectedEntityAttr.stringValue();
		log.debug("The selected entity name is "+selectedEntityName);
		if (!_entityList.isEmpty())
		{ 
			
			if (selectedEntityName != null &&
					!selectedEntityName.trim().equals("")) {
				// already has a selected entity in momol
				log.debug("There is a selected entity in actor");
				int selectedIndex = lookupEntityListName(selectedEntityName);
				if (selectedIndex == INDEXFORNOTFOUND) {
					throw new IllegalActionException(
					"The selected Entity in momol cound't be found");
				} else {
					_selectedTableEntity = (Entity) _entityList.elementAt(
							selectedIndex);
					_selectedCachedDataItem = (EcogridDataCacheItem) 
					_selectedTableEntity.getDataCacheObject();
					
				}
			} else {
				// no selected enity in momol and we need selected one
				log.debug("There is NOT a selected entity in actor");
				_selectedTableEntity = (Entity) _entityList.elementAt(
						DEFAULTINDEX);
				_selectedCachedDataItem = (EcogridDataCacheItem)_selectedTableEntity.getDataCacheObject();
				
				String entityName = _selectedTableEntity.getName();
				log.debug("set the default entity name " + entityName +
				"because the there is no selected entity");
				_selectedEntityAttr.setExpression(entityName);
				if(!fromAttributeChange)
				{
					log.debug("send a moml request for adding selected Entity parameter");
					StringBuffer buffer = new StringBuffer();
					buffer.append("<property name=\"");
					buffer.append(_SELECTEDENTITY);
					buffer.append("\" class=\"");
					buffer.append(_selectedEntityAttr.getClassName());
					buffer.append("\" value=\"");
					buffer.append(entityName);
					buffer.append("\"/>");
					String moml = buffer.toString();
					log.debug("The moml string is "+moml);
					NamedObj container = (NamedObj) this.getContainer();
					NamedObj composite = (NamedObj) container.getContainer();
					MoMLChangeRequest request = new MoMLChangeRequest(this, this, moml.toString());
					request.setUndoable(true);
					this.requestChange(request);
					
				}
			}
		}      
	}
	
	/*
	 * Method to find the entity index in EntityList which has the same name to the
	 * given string. If no entity index found, -1 will be returned.
	 */
	private int lookupEntityListName(String givenString)
	{
		log.debug("Looking for entity named " + givenString);
		int index = INDEXFORNOTFOUND;
		if (givenString != null && !givenString.trim().equals(""))
		{
			int size = _entityList.size();
			for (int i=0; i<size; i++)
			{
				Entity entity = (Entity)_entityList.elementAt(i);
				String entityName = entity.getName();
				if (entityName != null && !entityName.trim().equals("") && entityName.equals(givenString))
				{
					index = i;
					
				}
			}
		}
		log.debug("The selected index is "+index);
		return index;
	}
	
	/*
	 * This method will generate selected cloumn list by given a 
	 */
	private void generteLabelListAndTypteListFromColumns()
	{
		
		if (_columns != null)
		{
			int size = _columns.size();
			_selectedColumnLabelList = new String[size];
			_selectedColumnTypeList  = new Type[size];
			for(int i=0; i<size; i++)
			{
				DSTableFieldIFace column = (DSTableFieldIFace) _columns.elementAt(i);
				_selectedColumnLabelList[i] = column.getName();
				String type = column.getDataType();
				_selectedColumnTypeList[i] = getBaseType(type);
				
			}
		}
		
	}
	
	/*
	 * Method to transform a string array to token array base on given type
	 * 
	 */
	static Token[] transformStringVectorToTokenArray(Vector stringVector, Type type, Vector missingValuecode)
	throws IllegalActionException
	{
		if (stringVector == null)
		{
			return null;
		}
		int size = stringVector.size();
		Token[] columnToken    = new Token[size];
		for (int j =0; j<size; j++)
		{    
			String eleStr = (String)stringVector.elementAt(j); 
			log.debug("The column value "+eleStr);
			Token val = transformStringToToken(eleStr, type, missingValuecode);
			columnToken[j] = val;
		}
		
		return columnToken;
	}
    
	/*
	 * Method to transform a string  to token  base on given type and missingValue
	 * 
	 */
	static Token transformStringToToken(String eleStr, Type type, Vector missingValue)
	throws IllegalActionException
	{
		
		
		Token val = null;
		if (missingValue != null && !missingValue.isEmpty())
		{
			if (missingValue.contains(eleStr))
			{
			   eleStr = null;
			}
		}
		
		//find the data type for each att
		if (type == BaseType.INT) {
			try {
				if (eleStr != null)
				{
				   val = new IntToken(new Integer(eleStr).intValue());
				}
				else
				{
				   val = new IntToken(eleStr);
				   //val.nil();
				}
			} catch (Exception e) {
				val = new IntToken(0);
			}
		} else if (type == BaseType.DOUBLE) {
			if (eleStr != null)
			{
			   val = new DoubleToken(new Double(eleStr).doubleValue());
			}
			else
			{
			   val = new DoubleToken(eleStr);
			}
			
		} else if (type == BaseType.LONG) {
			if (eleStr != null)
			{
			   val = new LongToken(new Long(eleStr).longValue());
			}
			else
			{
				val = new LongToken(eleStr);
				//val.nil();
			}
			
		} else if (type == BaseType.STRING) {
			if (eleStr != null)
			{
			  val = new StringToken(eleStr);
			}
			else
			{
				eleStr = "nil";
				val = new StringToken(eleStr);
				//val.nil();
			}
			
		} else {
			val = new IntToken(0);
		}
		return val;
	}


    /**
     * Indicate to this that the workflow is being stopped.  This method is executed
     * from the Director when the user presses the "stop" button.  All this does
     * is release the execution thread if it happens to be blocked in the initialize
     * method waiting for _metaDataCompleted or _entityCountDown to complete.
     */
    public void stop() {
        log.debug("Stopping");
        
        synchronized( _metadataCompleted ) {
            _metadataCompleted.notifyAll();
        }
        synchronized( _entityCountDown ) {
            _entityCountDown.notifyAll();
        }
        // TODO Auto-generated method stub
        super.stop();
    }
    
    /**
     * This method will detemine if is end of resultset
     * @return
     */
    public boolean isEndOfResultset () throws SQLException
    {
    	if (  _resultSet != null && _resultSet.getResultSet().isAfterLast())
    	{
    		return true;
    	}
    	else
    	{
    		return false;
    	}
    	
    }

}
