/*
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.sdm.spa;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.StringToken;
import ptolemy.data.type.BaseType;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;

import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.DataInputStream;
import java.io.DataOutputStream;

/** Makes an http request to ClustalW web site.
 * It executes after all the promoter sequences are retrieved.
 * Returns the physical alignment sequence of all the gene sequences
 * in the input list.
 * @version     1.1
 * @author	Ilkay Altintas, Zhengang Cheng, Xiaowen Xin
 */

public class ClustalW extends TypedAtomicActor {

	public ClustalW(CompositeEntity aContainer, String aName)
		throws IllegalActionException, NameDuplicationException {

		super(aContainer, aName);
		result = new TypedIOPort(this, "result", false, true);
		result.setTypeEquals(BaseType.STRING);
		input = new TypedIOPort(this, "input", true, false);
		input.setTypeEquals(BaseType.STRING);
	}

	///////////////////////////////////////////////////////////////////
	////                     ports and parameters                  ////

	// -- Part of the Actor

	public TypedIOPort result;
	public TypedIOPort input;

	///////////////////////////////////////////////////////////////////
	////                         public methods                    ////

	/** Initialize the PN actor.
	 *  @exception IllegalActionException If the parent class throws it.
	 */
	public void initialize() throws IllegalActionException {
		super.initialize();
		_returnValue = true;
	}


	/** Fire the actor.
	 *  @exception IllegalActionException If there is no director.
	 */
	public void fire() throws IllegalActionException {
		inputString = ((StringToken)(input.get(0))).stringValue();
		_debug("Process input with length" + inputString.length());
		String res = "";
		res = this.alignSequences(inputString);
		if ( res.length() > 0) {

			result.broadcast( new StringToken(res));
		}
	}



	/** Post fire the actor. Return false to indicated that the
	 *  process has finished. If it returns true, the process will
	 *  continue indefinitely.
	 */
	public boolean postfire() {
		return true;
	}

	private String alignSequences(String cwData) {
		String seq = "";
		String line = "";
		try {
			URL url =
				new URL("http://clustalw.genome.ad.jp//sit-bin/nph-clustalw");
			URLConnection urlconn = url.openConnection();
			urlconn.setDoInput(true);
			urlconn.setDoOutput(true);
			urlconn.setUseCaches(false);
			urlconn.setRequestProperty(
				"Content-Type",
				"application/x-www-form-urlencoded");
			DataOutputStream ostream =
				new DataOutputStream(urlconn.getOutputStream());
			String query = "output="
					+ URLEncoder.encode("clustal", "UTF-8")
					+ "&pwalignment="
					+ URLEncoder.encode("FAST/APPROXIMATE", "UTF-8")
					+ "&type="
					+ URLEncoder.encode("dna", "UTF-8")
					+ "&sequence="
					+ URLEncoder.encode(cwData, "UTF-8");
			//System.out.println(query);
			byte[] qv = query.getBytes();
			ostream.write(qv);
			ostream.flush();
			ostream.close();

			DataInputStream inStream =
				new DataInputStream(urlconn.getInputStream());
			BufferedReader inReader =
				new BufferedReader(
					new InputStreamReader(urlconn.getInputStream()));

			while (null != ((line = inReader.readLine())))
				seq += line + "\n";

			inReader.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return seq;
	}

	///////////////////////////////////////////////////////////////////
	////                         private variables                 ////

	// -- Get private copies of the parameters
	private String inputString;

	private boolean _returnValue = true;

}
